# -*- coding: utf-8 -*-
from PyQt5 import *
from PyQt5 import QtGui
from PyQt5 import QtWidgets
import common
import dialogbrowserui
import logging

class dialogbrowserUI(QtWidgets.QDialog, dialogbrowserui.Ui_DialogBrowser):

    connectionname = ""

    def __init__(self, name, configfile):
        super(self.__class__, self).__init__()
        self.setupUi(self)
        self.connectionname = name
        # label
        common.setRessourceFile(configfile)
        self.setWindowTitle(common.getRessource("browserDialogTitle"))
        self.browserOKButton.setText(common.getRessource("ButtonOK"))
        self.browserCancelButton.setText(common.getRessource("ButtonCancel"))
        self.browserLabelName.setText(common.getRessource("configDialogLabelName"))
        self.browserLabelBrowser.setText(common.getRessource("browserDialogLabelBrowser"))
        self.browserLabelParameter.setText(common.getRessource("configDialogLabelParameter"))
        self.browserInputRepeat.setText(common.getRessource("configDialogLabelRepeat"))
        self.browserInputIcon.setText(common.getRessource("configDialogLabelIcon"))
        self.browserLabelIconName.setText(common.getRessource("configDialogLabelIconName"))
        # action
        self.browserOKButton.clicked.connect(self.ButtonOK)
        self.browserCancelButton.clicked.connect(self.ButtonCancel)
        # fill comboBox
        common.fillComboBox(self, "browserBrowsers", self.browserInputBrowser)
        
        if self.connectionname != "":
            # read connection parameter and fill dialog
            connection = common.readConnection(self.connectionname)
            self.browserInputName.setText(self.connectionname)
            # Browser
            browser = connection["browser"]
            if browser != "":
                index = self.browserInputBrowser.findText(browser, QtCore.Qt.MatchFixedString)
                self.browserInputBrowser.setCurrentIndex(index)
            self.browserInputParameter.setText(connection["parameter"])
            autostart = connection["autostart"]
            if autostart == "yes":
                self.browserInputAutostart.setChecked(True)
            # Repeat
            repeat = connection["repeat"]
            if repeat == "yes":
                self.browserInputRepeat.setChecked(True)
            # Icon
            icon = "no"
            if "icon" in connection:
                icon = connection["icon"]
            if icon == "yes":
                self.browserInputIcon.setChecked(True)
            iconname = ""
            if "iconname" in connection:
                iconname = connection["iconname"]
            self.browserInputIconName.setText(iconname)
            
    def ButtonOK(self):
        logging.info("ButtonOK")
        error = False
        if self.browserInputName.text() == "":
            common.messageDialog("configDialogErrorName")
            error = True
        elif self.browserInputIcon.isChecked() and self.browserInputIconName.text() == "":
            common.messageDialog("configDialogErrorIconName")
            error = True
            
        name = self.browserInputName.text()
        if common.existingConnection(name):
            common.messageDialog("configDialogErrorAddress")
            error = True

        if error == False:
            values = {}
            values["typ"] = "browser"
            values["name"] = self.browserInputName.text()
            index = self.browserInputBrowser.currentIndex()
            reslist = common.getRessourceList("browserCommand")
            values["browser"] = reslist[index]
            
            values["parameter"] = self.browserInputParameter.text()
            if self.browserInputAutostart.isChecked():
                values["autostart"] = "yes"
            else:
                values["autostart"] = "no"
            if self.browserInputRepeat.isChecked():
                values["repeat"] = "yes"
            else:
                values["repeat"] = "no"
            
            if self.browserInputIcon.isChecked():
                values["icon"] = "yes"
            else:
                values["icon"] = "no"
            
            values["iconname"] = str(self.browserInputIconName.text())
            
            # delete old connection
            if self.connectionname != "":
                common.deleteConnection(self.connectionname)
                common.deletePasswordFile(self.connectionname)
            # delete new connection
            common.deleteConnection(self.browserInputName.text())
            common.deletePasswordFile(self.browserInputName.text())
            # make new connection
            parameter = self.parameterbrowser(self.browserInputName.text(), values)
            common.newConnection(values, parameter, self.browserInputName.text())
            self.close()

    def ButtonCancel(self):
        logging.info("ButtonCancel")
        self.close()

    def parameterbrowser(self, connectionname, values):
        logging.info("parameterbrowser")
        parameters = values["browser"]
        if values["parameter"] != "":
            parameters = parameters + " " + values["parameter"]
        logging.info(parameters)
        return parameters
