#!/usr/bin/python3
import sys
import os
import logging
import common
import vpn

def configSystem(systemfile, verbose):
    if verbose:
        print("read file " + systemfile)
    values = readParameter(systemfile)
    if verbose:
        print(values)
    
    networksave = True
    networksave = False
    if "system_networksave" in values:
        if values["system_networksave"] == "yes":
            networksave = True
    if verbose:
        print("config network")
    if networksave:
        # write network
        networkvalues = {}
        if "network_interface" in values:
            networkvalues["connection"] = values["network_interface"]
        if "network_address" in values:
            networkvalues["address"] = values["network_address"]
        if "network_subnetmask" in values:
            networkvalues["subnetmask"] = values["network_subnetmask"]
        if "network_gateway" in values:
            networkvalues["gateway"] = values["network_gateway"]
        else:
            networkvalues["gateway"] = ""
        if "network_dns" in values:
            networkvalues["dns"] = values["network_dns"]
        else:
            networkvalues["dns"] = ""
        if "network_wlan_ssid" in values:
            networkvalues["ssid"] = values["network_wlan_ssid"]
        else:
            networkvalues["ssid"] = ""
        if "network_wlan_password" in values:
            networkvalues["password"] = values["network_wlan_password"]
        else:
            networkvalues["password"] = ""
            
        if verbose:
            print(networkvalues)
            
        if len(networkvalues) > 0:
            if values["network_typ"] == "dhcp":
                if verbose:
                    print(" set network")
                common.networkWriteDHCP(networkvalues)
            else:
                if verbose:
                    print(" set network")
                common.networkWriteStaticIP(networkvalues)
            if verbose:
                print("OK")
    
    if verbose:
        print("config vnc")
        
    if "system_vnc" in values:
        if verbose:
            print(" set vnc")
        if values["system_vnc"] == "on": 
            if values["system_vnc_password"] != "":
                common.remoteVNC(True, values["system_vnc_password"])
            else:
                common.remoteVNC(False, None)
    else:
        values["system_vnc"] = "off"
        
    if verbose:
        print("OK")
        print("config ssh")
        
    if "system_ssh" in values:
        if verbose:
            print(" set ssh")
        if values["system_ssh"] == "on":
            if values["system_ssh_password"] != "":
                common.remoteSSH(True, values["system_ssh_password"])
        else:
            common.remoteSSH(False, None)
    else:
        values["system_ssh"] = "off"
        
    if verbose:
        print("OK")
    
    # set up config scripts    
    # read system
    if verbose:
        print("config system")
    systemValues = common.readSystem()
    if not "startAdmin" in systemValues:
        systemValues["startAdmin"] = "yes"
    if not "language" in systemValues:
        systemValues["language"] = "English"
    if not "keyboardLayout" in systemValues:
        systemValues["keyboardLayout"] = "gb"
    if  not "startminimized" in values:
        systemValues["startMinimized"] = "no"
    if  not "numerickeypad" in values:
        systemValues["numerickeypad"] = "no"
    if  not "networkSave" in values:
        systemValues["networkSave"] = "yes"
    if  not "resolution" in values:
        systemValues["resolution"] = "auto"
    if  not "resolution2" in values:
        systemValues["resolution2"] = "auto"
    if not "simplemenu" in values:
        systemValues["simplemenu"] = "no"
    if not "virtualconsole" in values:
        systemValues["virtualconsole"] = "yes"    
        
    if "system_admin" in values:
        systemValues["startAdmin"] = values["system_admin"]
    if systemValues["startAdmin"] == "no":
        if "system_admin_password" in values:
            if values["system_admin_password"] != "":
                systemValues["adminPassword"] = common.convertPassword(values["system_admin_password"])
    if "system_language" in values:
        systemValues["language"] = values["system_language"]
    if "system_keyboard" in values:
        systemValues["keyboardLayout"] = values["system_keyboard"]
    if "system_startminimized" in values:
        systemValues["startMinimized"] = values["system_startminimized"]
    if "system_numerickeypad" in values:
        systemValues["numerickeypad"] = values["system_numerickeypad"]
    if "system_networkSave" in values:
        systemValues["networkSave"] = values["system_networksave"]
    if "monitor_resolution" in values:
        systemValues["resolution"] = values["monitor_resolution"]
        common.setResolution(values["monitor_resolution"])
    if "monitor_resolution2" in values:
        systemValues["resolution2"] = values["monitor_resolution2"]
        if "monitor_orientation" in values:
            common.setMultiMonitor(True, values["monitor_orientation"], values["monitor_resolution2"])
    if "security_simplemenu" in values:
        systemValues["simplemenu"] = values["security_simplemenu"]
    if "security_virtualconsole" in values:
        systemValues["virtualconsole"] = values["security_virtualconsole"]

    if verbose:
        print(systemValues)
    if len(systemValues) > 0:
        if verbose:
            print("set system")
        common.writeSystem(systemValues)
        
    if "system_name" in values:
        if verbose:
            print("system name: " + values["system_name"])
        common.setSystemname(values["system_name"])
        
    if "system_keyboard" in values:
        if verbose:
            print(" set keyboard")
        common.writeKeyboardLayout(values["system_keyboard"])
    if "monitor_screensaver" in values:
        if verbose:
            print(" set screensaver: " + values["monitor_screensaver"])
        common.setScreensaver(values["monitor_screensaver"])
    if "monitor_standby" in values:
        if verbose:
            print(" set monitor standby")
        common.setMonitorStandby(values["monitor_standby"])
    if "monitor_resolution" in values:
        if verbose:
            print(" set resolution")
        common.setResolution(values["monitor_resolution"])
    if "system_usbautomount" in values:
        if verbose:
            print(" set usb automount")
        common.setUSBAutomount(values["system_usbautomount"])
    if "system_oneninedesign" in values:
        if verbose:
            print(" set case oneninedesign")
        common.setOneninedesign(values["system_oneninedesign"])
    if "system_argon1" in values:
        if verbose:
            print(" set case argon1")
        common.setArgon1(values["system_argon1"])
    if "system_soundcard" in values:
        if verbose:
            print(" set soundcard")
        common.setSoundCard(values["system_soundcard"])
    if "security_root_password" in values:
        common.setRootPassword(values["security_root_password"])
    if "security_user_password" in values:
        common.setUserPassword(values["security_user_password"])
    if "monitor_optimizegraphic" in values:
        if verbose:
            print(" set optimizegraphic")
        graphic = False
        if values["monitor_optimizegraphic"] == "yes":
            graphic = True
        common.setOptimizeGraphic(graphic)
        
    if verbose:
        print("OK")
        print("config time")
        
    timeValues = common.readTime()
    if "time_save" in values:
        timeValues["save"] = values["time_save"]
    if "time_time" in values:
        timeValues["time"] = values["time_time"]
    timeValues["automatic"] = "no"
    if "time_automatic" in values:
        timeValues["automatic"] = values["time_automatic"]
    if "time_ntp" in values:
        timeValues["ntp"] = values["time_ntp"]
    if "time_timezone" in values:
        timeValues["timezone"] = values["time_timezone"]
    timeValues["execute"] = "no"
    if "time_execute" in values:
        timeValues["execute"] = values["time_execute"]
    timeValues["repeat"] = "no"
    if "time_repeat" in values:
        timeValues["repeat"] = values["time_repeat"]
    if "time_action" in values:
        timeValues["action"] = values["time_action"]
    if "time_at" in values:
        timeValues["at"] = values["time_at"]
    if "time_last" in values:
        timeValues["last"] = values["time_last"]
        
    if verbose:        
        print(timeValues)
        
    if len(timeValues) > 0:
        if verbose:
            print(" set time")
        common.writeTime(timeValues)

    timeValues = {}
    if "time_show" in values:
        timeValues["show"] = values["time_show"]
    if len(timeValues) > 0:
        common.writeTimeTemp(timeValues)
    
    if verbose:
        print("OK")
    
    # write vpn
    if verbose:
        print("config vpn")
    vpnvalues = {}
    if "vpn_program" in values:
        vpnvalues["program"] = values["vpn_program"]
    if "vpn_configfile" in values:
        vpnvalues["configfile"] = values["vpn_configfile"]
    if "vpn_parameter" in values:
        vpnvalues["parametervpn"] = values["vpn_parameter"]
    if "runfile" in values:
        vpnvalues["runfile"] = values["vpn_runfile"]
    if "vpn_autostart" in values:
        vpnvalues["autostartvpn"] = values["vpn_autostart"]
    if "vpn_systemlogin" in values:
        vpnvalues["systemlogin"] = values["vpn_systemlogin"]
    if "vpn_otp" in values:
        vpnvalues["otp"] = values["vpn_otp"]
    
    if verbose:
        print(vpnvalues)
    if len(vpnvalues) > 0:
        if verbose:
            print(" set vpn")
        vpn.writeVPN(vpnvalues)
    if verbose:
        print("OK")
    # config system
    if verbose:
        print("configure system")
    command = common.getRessource("commandSystemReboot")
    command = command + " > /tmp/system.mld 2> /tmp/system.err"
    common.runProgram(command)
    
# read system paramater file
def readParameter(filename):
    logging.info("readParameter " + filename)
    values = {}
    filer = open(filename, "r")
    for line in filer:
        line = line.strip()
        if line != "":
            stringlist = line.split("=", 2)
            name = stringlist[0]
            name = name.strip()
            value = stringlist[1]
            value = value.strip()
            values[name] = value
    filer.close()
    return values

if __name__ == "__main__":
    verbose = False
    if len(sys.argv) < 1:
        print("ERROR: file name missing")
        sys.exit(1)
        
    for index in range(len(sys.argv)):
        if sys.argv[index] == "-v":
            verbose = True
        else:
            fileName = sys.argv[index]
    
    configSystem(fileName, verbose)
    print("to complete configuration: reboot now system\n")