#!/usr/bin/python3
# -*- coding: utf-8 -*-
## -*- coding: iso-8859-1 -*-

# File:    thinclient.py
# Author:  Volker Matheis
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from PyQt5 import QtCore
from PyQt5 import QtGui
from PyQt5 import QtWidgets
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from PyQt5.QtCore import QThread
import logging
import os
import sys
import time
from datetime import datetime, timedelta
import common
import connectThread
import configurationdialog
import dialoglogin
import dialogpassword
import dialogpassworddecrypt
import dialogrdesktop
import dialogrdp
import dialogrealvnc
import dialogssh
import dialogtigervnc
import dialogtightvnc
import dialogx2go
import dialogxdmcp
import dialogbrowser
import thinclientui
import vpn


class thinClientUI(QtWidgets.QMainWindow, thinclientui.Ui_MainWindow):
    connectedThreads = {}
    
    def __init__(self, configfile):
        super(self.__class__, self).__init__()
        self.setupUi(self)
        self.configfile = configfile
        if self.configfile == "":
            self.configfile = "thinclient.conf"
        if not os.path.isfile(self.configfile):
            print("ERROR: Config file not found: " + self.configfile)
            sys.exit(1)

        # logging
        loggingfilename = common.getRessourceByName(self.configfile, "loggingFileThinclient")
        debugfile = common.getRessourceByName(self.configfile, "debugSwitch")
        debug = False
        if os.path.isfile(debugfile):
            debug = True
        common.loggingStart(loggingfilename, debug)
        # init system
        common.getSystemFile(self.configfile)
        common.setRessourceFile(self.configfile)
        systemLanguage = common.getLanguage()
        # delete connection log file
        common.deleteConnectionLog()
        # delete existing config file
        common.deleteConfigScript()

        # ask decrypt password data dir
        self.encryptPassword = ""
        if common.isEncryptData():
            result = False
            while(not result):
                dialogPasswordDecrypt = dialogpassworddecrypt.dialogPasswordUI(self.configfile)
                dialogPasswordDecrypt.exec_()
                result = dialogPasswordDecrypt.getResult()
                if result:
                    self.encryptPassword = dialogPasswordDecrypt.getPassword()
                    result = common.decryptData(self.encryptPassword)
                else:
                    sys.exit(1)
        
        # label
        self.connectButton.setText(common.getRessource("connect"))
        self.connectButtonCancel.setText(common.getRessource(
                                         "connectButtonCancel"))
        self.connectButtonExtended.setText(common.getRessource(
                                           "connectButtonExtended"))
        self.systemButtonReboot.setText(common.getRessource(
                                        "systemButtonReboot"))
        self.systemButtonShutdown.setText(common.getRessource(
                                          "systemButtonShutdown"))
        self.connectButtonLog.setText(common.getRessource(
                                          "connectButtonLog"))
        self.configButtonNew.setText(common.getRessource(
                                     "configButtonNew"))
        self.configButtonEdit.setText(common.getRessource(
                                      "configButtonEdit"))
        self.configButtonDelete.setText(common.getRessource(
                                        "configButtonDelete"))
        self.networkInputSave.setText(common.getRessource(
                                           "networkInputSave"))
        self.networkLabelInterface.setText(common.getRessource(
                                           "networkLabelInterface"))
        self.networkInputDHCP.setText(common.getRessource("networkInputDHCP"))
        self.networkInputStaticIP.setText(common.getRessource(
                                          "networkInputStaticIP"))
        self.networkLabelAddress.setText(common.getRessource(
                                         "networkLabelAddress"))
        self.networkLabelSubnetmask.setText(common.getRessource(
                                            "networkLabelSubnetmask"))
        self.networkLabelGateway.setText(common.getRessource(
                                         "networkLabelGateway"))
        self.networkLabelDNS.setText(common.getRessource("networkLabelDNS"))
        self.networkLabelWlanSSID.setText(common.getRessource("networkLabelWlanSSID"))
        self.networkInputSearchSSID.setText(common.getRessource("networkInputSearchSSID"))
        self.networkLabelWlanPassword.setText(common.getRessource("networkLabelWlanPassword"))
        self.systemLabelSystemname.setText(common.getRessource(
                                          "systemLabelSystemname"))
        self.systemButtonTerminal.setText(common.getRessource(
                                          "systemButtonTerminal"))
        self.systemInputMinimize.setText(common.getRessource(
                                         "systemInputMinimize"))
        self.systemInputNumericKeypad.setText(common.getRessource(
                                         "systemInputNumericKeypad"))
        self.systemLabelLanguage.setText(common.getRessource(
                                         "systemLabelLanguage"))
        self.systemLabelKeyboardLayout.setText(common.getRessource(
                                               "systemLabelKeyboardLayout"))
        self.systemLabelPassword.setText(common.getRessource(
                                         "systemLabelPassword"))
        self.systemInputRemoteVNC.setText(common.getRessource(
                                          "systemInputRemoteVNC"))
        self.systemLabelRemotePassword.setText(common.getRessource(
                                               "systemLabelRemotePassword"))
        self.systemInputSSH.setText(common.getRessource(
                                    "systemInputSSH"))
        self.systemLabelsshPassword.setText(common.getRessource(
                                         "systemLabelsshPassword"))
        self.systemInputUSBAutomount.setText(common.getRessource(
                                    "systemInputUSBAutomount"))
        self.systemInputOneninedesign.setText(common.getRessource(
                                    "systemInputOneninedesign"))
        self.systemInputArgon1.setText(common.getRessource(
                                    "systemInputArgon1"))
        self.systemInputEncrypt.setText(common.getRessource(
                                    "systemInputEncrypt"))
        self.systemLabelPasswordEncrypt.setText(common.getRessource(
                                    "systemLabelPasswordEncrypt"))
        self.systemLabelPasswordRepeatEncrypt.setText(common.getRessource(
                                    "systemLabelPasswordRepeatEncrypt"))
        self.systemInputPrinterConfig.setText(common.getRessource(
                                    "systemInputPrinterConfig"))
        self.systemInputScreensaver.setText(common.getRessource(
                                            "systemScreensaver"))
        self.systemInputMonitorStandby.setText(common.getRessource(
                                               "systemMonitorStandby"))
        self.systemLabelResolution.setText(common.getRessource(
                                           "systemLabelResolution"))
        self.systemLabelResolution2.setText(common.getRessource(
                                           "systemLabelResolution2"))
        self.systemInputOptimizeGraphic.setText(common.getRessource(
                                            "systemOptimizeGraphic"))
        self.vpnInputSystemLogin.setText(common.getRessource(
                                         "configDialogLabelSystemLogin"))
        self.vpnInputAutostart.setText(common.getRessource(
                                       "configDialogLabelAutostart"))
        self.vpnButtonConnect.setText(common.getRessource(
                                      "vpnButtonConnect"))
        self.vpnButtonCancel.setText(common.getRessource(
                                     "vpnButtonCancel"))
        self.vpnInputOvpn.setText(common.getRessource(
                                        "vpnInputOvpn"))
        self.vpnInputWireguard.setText(common.getRessource(
                                        "vpnInputWireguard"))
        self.vpnInputOTP.setText(common.getRessource(
                                        "vpnInputOTP"))
        self.vpnLabelParameter.setText(common.getRessource(
                                       "configDialogLabelParameter"))
        self.vpnLabelRunfile.setText(common.getRessource(
                                       "vpnLabelRunfile"))
        self.vpnButtonCondition.setText(common.getRessource(
                                        "vpnButtonCondition"))
        self.vpnButtonCondition.setIcon(QtGui.QIcon(common.getRessource("disconnectIcon")))
        self.vpnButtonFiles.setText(common.getRessource(
                                    "vpnButtonAdditionalFiles"))
        self.vpnButtonDelete.setText(common.getRessource(
                                    "vpnButtonDelete"))
        self.saveButtonSave.setText(common.getRessource(
                                    "saveButtonSave"))
        self.systemLabelSound.setText(common.getRessource(
                                    "systemLabelSound"))
        self.systemInputMultiMonitor.setText(common.getRessource(
                                    "systemInputMultiMonitor"))
        self.systemInputMultiMonitorLeft.setText(common.getRessource(
                                    "systemInputMultiMonitorLeft"))
        self.systemInputMultiMonitorRight.setText(common.getRessource(
                                    "systemInputMultiMonitorRight"))
        self.systemInputMultiMonitorAbove.setText(common.getRessource(
                                    "systemInputMultiMonitorAbove"))
        self.systemInputMultiMonitorBelow.setText(common.getRessource(
                                    "systemInputMultiMonitorBelow"))
        self.systemInputTaskbar.setText(common.getRessource(
                                    "systemInputTaskbar"))
        self.systemInputControlSound.setText(common.getRessource(
                                    "systemInputControlSound"))
        self.systemLabelBackgroundImage.setText(common.getRessource(
                                    "systemLabelBackgroundImage"))
        self.systemLabelBackgroundColor.setText(common.getRessource(
                                    "systemLabelBackgroundColor"))
        self.timeInputSave.setText(common.getRessource(
                                    "timeInputSave"))
        self.timeInputAuto.setText(common.getRessource(
                                    "timeInputAuto"))
        self.timeInputView.setText(common.getRessource(
                                    "timeInputView"))
        self.timeLabelTime.setText(common.getRessource(
                                    "timeLabelTime"))
        self.timeLabelNTP.setText(common.getRessource(
                                    "timeLabelNTP"))
        self.timeLabelActionGroup.setTitle(common.getRessource(
                                    "timeLabelActionGroup"))
        self.timeInputAction.setText(common.getRessource(
                                    "timeInputAction"))
        self.timeLabelActionAction.setText(common.getRessource(
                                    "timeLabelActionAction"))
        self.timeLabelActionTime.setText(common.getRessource(
                                    "timeLabelActionTime"))
        self.timeInputActionRepeat.setText(common.getRessource(
                                    "timeInputActionRepeat"))
        self.timeLabelActionLast.setText(common.getRessource(
                                    "timeLabelActionLast"))
        self.timeLabelTimezone.setText(common.getRessource(
                                    "timeLabelTimezone"))
        self.securityLabelRootPassword.setText(common.getRessource(
                                    "securityLabelRootPassword"))
        self.securityLabelUserPassword.setText(common.getRessource(
                                    "securityLabelUserPassword"))
        self.securityInputMenu.setText(common.getRessource(
                                    "securityInputMenu"))
        self.securityInputVTerminal.setText(common.getRessource(
                                    "securityInputVTerminal"))
        
        self.timeInputSave.setChecked(True)
        self.timeInputTime.setDisplayFormat("yyyy-MM-dd HH:mm") 
        # get current time and show
        self.timeInputTime.setDateTime(QtCore.QDateTime.currentDateTime())
        
        pixmap = QPixmap(common.getRessource("connectIcon"))
        self.systemLabelInfoImage.setPixmap(pixmap)
        self.thinClientTab.setTabText(0, common.getRessource("tabConnect"))
        self.thinClientTab.setTabText(1, common.getRessource("tabSystem"))
        self.thinClientTab.setTabText(2, common.getRessource("tabConfig"))
        self.thinClientTab.setTabText(3, common.getRessource("tabNetwork"))
        self.thinClientTab.setTabText(4, common.getRessource("tabVPN"))
        self.thinClientTab.setTabText(5, common.getRessource("tabSettings"))
        self.thinClientTab.setTabText(6, common.getRessource("tabHardware"))
        self.thinClientTab.setTabText(7, common.getRessource("tabMonitor"))
        self.thinClientTab.setTabText(8, common.getRessource("tabTime"))
        self.thinClientTab.setTabText(9, common.getRessource("tabSecurity"))
        self.thinClientTab.setTabText(10, common.getRessource("tabSave"))

        # widget actions
        self.connectButton.clicked.connect(self.connection)
        self.connectButtonCancel.clicked.connect(self.connectionCancel)
        self.connectButtonExtended.clicked.connect(self.connectionExtended)
        self.configButtonNew.clicked.connect(self.configNewConnection)
        self.configButtonEdit.clicked.connect(self.configEdit)
        self.configButtonDelete.clicked.connect(self.configDelete)
        self.networkInputDHCP.toggled.connect(self.networkType)
        self.networkInputStaticIP.toggled.connect(self.networkType)
        self.systemButtonReboot.clicked.connect(self.systemReboot)
        self.systemButtonShutdown.clicked.connect(self.systemShutdown)
        self.connectButtonLog.clicked.connect(self.connectionLog)
        self.connectList.doubleClicked.connect(self.connection)
        self.configList.doubleClicked.connect(self.configListItem)
        self.systemButtonTerminal.clicked.connect(self.systemTerminal)
        self.vpnInputOvpn.clicked.connect(self.vpnClearGetFile)
        self.vpnInputWireguard.clicked.connect(self.vpnClearGetFile)
        self.vpnButtonFile.clicked.connect(self.vpnGetFile)
        self.vpnButtonRunfile.clicked.connect(self.vpnRunFile)
        self.vpnButtonDelete.clicked.connect(self.vpnDeleteFile)
        self.vpnButtonConnect.clicked.connect(self.vpnConnect)
        self.vpnButtonCancel.clicked.connect(self.vpnCancel)
        self.vpnButtonCondition.clicked.connect(self.vpnCondition)
        self.vpnButtonFiles.clicked.connect(self.vpnAdditionalFiles)
        self.systemInputArgon1.toggled.connect(self.systemCaseArgon1)
        self.systemInputOneninedesign.toggled.connect(self.systemCaseOneninedesign)
        self.saveButtonSave.clicked.connect(self.save)
        self.systemInputRemotePassword.textEdited.connect(self.remotePasswordChanged)
        self.systemInputPassword.textEdited.connect(self.passwordChanged)
        self.systemInputsshPassword.textEdited.connect(self.sshPasswordChanged)
        self.securityInputRootPassword.textEdited.connect(self.rootPasswordChanged)
        self.securityInputUserPassword.textEdited.connect(self.userPasswordChanged)
        self.systemInputPasswordEncrypt.textEdited.connect(self.encryptPasswordChanged)
        self.networkInputSearchSSID.clicked.connect(self.searchSSID)
        self.systemButtonMode.clicked.connect(self.setMode)
        self.systemInputPrinterConfig.clicked.connect(self.printer)
        self.networkInputWlanPassword.textEdited.connect(self.wlanPasswordChanged)
        self.systemButtonBackgroundImageFile.clicked.connect(self.backgroundImageGetFile)

        # show/hide password
        self.networkInputWlanPasswordShow.clicked.connect(self.showWlanPassword)
        self.systemInputPasswordShow.clicked.connect(self.showPassword)
        self.systemInputRemotePasswordShow.clicked.connect(self.showRemotePassword)
        self.systemInputsshPasswordShow.clicked.connect(self.showsshPassword)
        self.securityInputRootPasswordShow.clicked.connect(self.showRootPassword)
        self.securityInputUserPasswordShow.clicked.connect(self.showUserPassword)
        
        # edit network?
        self.networkInputInterface.activated.connect(self.networkChanged)
        self.networkInputStaticIP.toggled.connect(self.networkChanged)
        self.networkInputDHCP.toggled.connect(self.networkChanged)
        self.networkInputAddress.textChanged.connect(self.networkChanged)
        self.networkInputSubnetmask.textChanged.connect(self.networkChanged)
        self.networkInputGateway.textChanged.connect(self.networkChanged)
        self.networkInputDNS.textChanged.connect(self.networkChanged)
        self.networkInputWlanSSID.activated.connect(self.networkChanged)
        self.networkInputWlanPassword.textChanged.connect(self.networkChanged)
        
        # set password to "not changed"
        self.wlanPasswordChanged = False
        self.RemotePasswordChanged = False
        self.PasswordChanged = False
        self.sshPasswordChanged = False
        self.RootPasswordChanged = False
        self.UserPasswordChanged = False
        self.EncryptPasswordChanged = False
        self.NetworkInputWlanPasswordShow = False
        self.SystemInputPasswordShow = False
        self.SystemInputRemotePasswordShow = False
        self.SystemInputsshPasswordShow = False
        self.SecurityInputRootPasswordShow = False
        self.SecurityInputUserPasswordShow = False
        
        # get build
        common.getBuild(self)
        # init security password fields
        self.securityInputRootPassword.setPlaceholderText("●●●●●●●●●●")
        self.securityInputUserPassword.setPlaceholderText("●●●●●●●●●●")
        
        value = common.getMode()
        if value == "admin":
            self.connectButtonExtended.setEnabled(False)
            
        # read network        
        interfaces = common.readConnectionInterfaces()
        for interface in interfaces:
            self.networkInputInterface.addItem(interface)
            
        for interface in interfaces:
            if common.isNetworkInterfaceUp(interface):
                index = self.networkInputInterface.findText(interface,
                                                            QtCore.Qt.MatchFixedString)
                self.networkInputInterface.setCurrentIndex(index)
            
        # get wlan devices
        # don't search on boot
        #self.searchSSID()
            
        values = common.readNetwork(self)
        if len(values) == 0:
            self.networkInputSave.setChecked(True)
            
        dhcp = True
        if "dhcp" in values:
            dhcp = values["dhcp"]
        if "address" in values:
            self.networkInputAddress.setText(values["address"])
        if "netmask" in values:
            self.networkInputSubnetmask.setText(values["netmask"])
        if "gateway" in values:
            self.networkInputGateway.setText(values["gateway"])
        if "dns" in values:
            self.networkInputDNS.setText(values["dns"])
        if "ssid" in values:
            index = self.networkInputWlanSSID.findText(values["ssid"], QtCore.Qt.MatchFixedString)
            if index >= 0:
                self.networkInputWlanSSID.setCurrentIndex(index)
            else:
                if values["ssid"] != "":
                    self.networkInputWlanSSID.addItem(values["ssid"])
                    index = self.networkInputWlanSSID.findText(values["ssid"], QtCore.Qt.MatchFixedString)
                    self.networkInputWlanSSID.setCurrentIndex(index)    
        
        values = common.getCurrentIP()
        if "address" in values:
            if values["address"] != "":
                self.networkInputAddress.setText(values["address"])
        if "netmask" in values:
            if values["netmask"] != "":
                self.networkInputSubnetmask.setText(values["netmask"])
        if "gateway" in values:
            if values["gateway"] != "":
                self.networkInputGateway.setText(values["gateway"])
        
        if dhcp:
            self.networkInputDHCP.setChecked(True)
            self.networkInputStaticIP.setChecked(False)
        else:
            self.networkInputDHCP.setChecked(False)
            self.networkInputStaticIP.setChecked(True)

        # fill ComboBox with connection types
        common.fillComboBox(self, "configConnectionType",
                            self.configInputConnectionType)
        
        # get system name
        systemname = common.getSystemname()
        self.systemInputSystemname.setText(systemname)
        
        # get language
        common.fillComboBox(self, "language", self.systemInputLanguage)
        if systemLanguage == "DE":
            language = "Deutsch"
            index = self.systemInputLanguage.findText(language,
                                                      QtCore.Qt.MatchFixedString)
            self.systemInputLanguage.setCurrentIndex(index)
        if systemLanguage == "IT":
            language = "Italiano"
            index = self.systemInputLanguage.findText(language,
                                                      QtCore.Qt.MatchFixedString)
            self.systemInputLanguage.setCurrentIndex(index)
        # get keyboard layout
        values = common.getKeyboardLayouts()
        common.fillComboBoxList(self, values, self.systemInputKeyboardLayout)
        layout = common.getCurrentKeyboardLayout()
        index = self.systemInputKeyboardLayout.findText(layout,
                                                        QtCore.Qt.MatchFixedString)
        if index >= 0:
            self.systemInputKeyboardLayout.setCurrentIndex(index)

        # Get Resolution List
        resolution, resolution2 = common.getResolutionList()
        common.fillComboBoxList(self, resolution, self.systemInputResolution)
        common.fillComboBoxList(self, resolution2, self.systemInputResolution2)
        
        # get orientation 
        orientation = common.getOrientationMonitor2()
        if orientation == "left":
            self.systemInputMultiMonitorLeft.setChecked(True)
        if orientation == "right":
            self.systemInputMultiMonitorRight.setChecked(True)
        if orientation == "above":
            self.systemInputMultiMonitorAbove.setChecked(True)
        if orientation == "below":
            self.systemInputMultiMonitorBelow.setChecked(True)

        # Check vc4-fkms-v3d config.txt
        system = common.getSystemTyp()
        if system == "PI4":
            self.systemInputMultiMonitor.setEnabled(False)
            self.systemInputMultiMonitor.setChecked(True)
        if system == "PI3":
            self.systemInputMultiMonitor.setEnabled(False)
            self.systemInputMultiMonitorGroupBox.setEnabled(False)
            self.systemInputOptimizeGraphic.setChecked(True)
            if common.isOptimizeGraphic():
                self.systemInputOptimizeGraphic.setChecked(False)
        else:
            self.systemInputOptimizeGraphic.setEnabled(False)
        
        # Check VNC
        if common.isRemoteVNC():
            self.systemInputRemoteVNC.setChecked(True)
            self.systemInputRemotePassword.setPlaceholderText("●●●●●●●●●●")
        # Check SSH
        if common.isRemoteSSH():
            self.systemInputSSH.setChecked(True)
            self.systemInputsshPassword.setPlaceholderText("●●●●●●●●●●")
        # check USB automount
        if common.isUSBAutomount():
            self.systemInputUSBAutomount.setChecked(True)

        # set time parameter
        if common.isDateShow():
            self.timeInputView.setChecked(True)
        # list time zones
        values = common.getTimezones()
        common.fillComboBoxList(self, values, self.timeInputTimezone)
        # fill action
        common.fillComboBox(self, "timeActions", self.timeInputActionAction)
        
        # is date and time showing?
        value = common.isDateShow()
        if value:
            self.timeInputView.setChecked(True)
            
        # get time parameter
        timeValues = common.readTime()
        for value in timeValues:
            if "save" in value:
                if timeValues["save"] == "no":
                    self.timeInputSave.setChecked(True)
                else:
                    self.timeInputSave.setChecked(False)
            if "automatic" in value:
                if timeValues["automatic"] == "yes":
                    self.timeInputAuto.setChecked(True)
            if "ntp" in value:
                self.timeInputNTP.setText(timeValues["ntp"])
            if "zone" in value:
                index = self.timeInputTimezone.findText(timeValues["zone"], QtCore.Qt.MatchFixedString)
                if index < 0:
                    index = 0
                self.timeInputTimezone.setCurrentIndex(index)
            if "time" in value:
                time = datetime.strptime(timeValues["time"], "%Y-%m-%d %H:%M")
                self.timeInputTime.setDateTime(time)
            if "execute" in value:
                if timeValues["execute"] == "yes":
                    self.timeInputAction.setChecked(True)
            if "repeat" in value:
                if timeValues["repeat"] == "yes":
                    self.timeInputActionRepeat.setChecked(True)
            if "action" in value:
                index = self.timeInputActionAction.findText(timeValues["action"], QtCore.Qt.MatchFixedString)
                self.timeInputActionAction.setCurrentIndex(index)
            if "execat" in value:
                if timeValues["execat"].startswith("+"):
                    self.timeInputActionTime.setText(timeValues["execat"])
                else:
                    if timeValues["execat"] != "":
                        time = datetime.strptime(timeValues["execat"], "%Y-%m-%d %H:%M")
                        strtime = time.strftime("%Y-%m-%d %H:%M")
                        self.timeInputActionTime.setText(strtime)
            if "last" in value:
                self.timeInputActionLast.setText(timeValues["last"])
        
        autostartvpn = False
        values = common.readSystem()
        if "adminPassword" in values:
            self.systemInputPassword.setPlaceholderText("●●●●●●●●●●")
        
        # user or admin mode?
        self.systemButtonMode.setText(common.getRessource("systemButtonModeAdmin"))
        self.startadmin = False
        value = common.getMode()
        if value == "admin":
            self.startadmin = True
            self.systemButtonMode.setText(common.getRessource("systemButtonModeUser"))
                
        # minimize dialog?
        if "startMinimized" in values:
            if values["startMinimized"] == "yes":
                self.systemInputMinimize.setChecked(True)
                self.showMinimized()
        # save network
        if "networkSave" in values:
            if values["networkSave"] == "no":
                self.networkInputSave.setChecked(True)
                
        # get current resolution
        resolution, resolution2 = common.getResolution()
        # set resolution monitor 1
        if "resolution" in values:
            if values["resolution"] == "auto":
                resolution = "auto"
        index = self.systemInputResolution.findText(resolution,
                                                        QtCore.Qt.MatchFixedString)
        if index >= 0:
            self.systemInputResolution.setCurrentIndex(index)
        else:
            if resolution != "":
                self.systemInputResolution.addItem(resolution)
                
        # set resolution monitor 2
        if "resolution2" in values:
            if values["resolution2"] == "auto":
                resolution2 = "auto"

        index = self.systemInputResolution2.findText(resolution2,
                                                         QtCore.Qt.MatchFixedString)
        if index >= 0:
            self.systemInputResolution2.setCurrentIndex(index)
        else:
            if resolution != "":
                self.systemInputResolution2.addItem(resolution2)
        
        if "multimonitor" in values:
            if values["multimonitor"] == "yes":
                self.systemInputMultiMonitor.setChecked(True)

        if "simplemenu" in values:
            if values["simplemenu"] == "yes":
                self.securityInputMenu.setChecked(True)
                
        if "virtualconsole" in values:
            if values["virtualconsole"] == "no":
                self.securityInputVTerminal.setChecked(True)
        
        if "numerickeypad" in values:
            if values["numerickeypad"] == "yes":
                self.systemInputNumericKeypad.setChecked(True)

        if common.isEncryptData():
            self.systemInputEncrypt.setChecked(True)
            self.systemInputPasswordEncrypt.setText(self.encryptPassword)
            self.systemInputPasswordRepeatEncrypt.setText(self.encryptPassword)
                
        # get background color
        backgroundcolor = common.getBackgroundColor()
        self.systemInputBackgroundColor.setText(backgroundcolor)
        
        # get background image
        backgroundimage = common.getBackgroundImage()
        self.systemInputBackgroundImage.setText(backgroundimage)
                
        # vpn
        vpnvalues = vpn.readVPN(self)
        self.vpnInputOvpn.setChecked(True)
        if "program" in vpnvalues:
            if vpnvalues["program"] == "wireguard":
                self.vpnInputWireguard.setChecked(True)
        if "configfile" in vpnvalues:
            self.vpnInputFile.setText(vpnvalues["configfile"])
        if "parametervpn" in vpnvalues:
            self.vpnInputParameter.setText(vpnvalues["parametervpn"])
        if "runfile" in vpnvalues:
            self.vpnInputRunfile.setText(vpnvalues["runfile"])
        if "systemlogin" in vpnvalues:
            if vpnvalues["systemlogin"] == "yes":
                self.vpnInputSystemLogin.setChecked(True)
        if "autostartvpn" in vpnvalues:
            if vpnvalues["autostartvpn"] == "yes":
                self.vpnInputAutostart.setChecked(True)
                autostartvpn = True
        if "otp" in vpnvalues:
            if vpnvalues["otp"] == "yes":
                self.vpnInputOTP.setChecked(True)
                
        if self.startadmin is True:
            common.enableTabsAdmin(self)
        else:
            common.enableTabsUser(self)
        self.thinClientTab.setCurrentIndex(0)
        
        # get screensaver state
        if common.isScreensaver():
            self.systemInputScreensaver.setChecked(True)
        else:
            self.systemInputScreensaver.setChecked(False)
            
        # get monitor standby state
        if common.isMonitorStandby():
            self.systemInputMonitorStandby.setChecked(True)
        else:
            self.systemInputMonitorStandby.setChecked(False)

        # get taskbar state
        if common.isTaskbar():
            self.systemInputTaskbar.setChecked(True)
        else:
            self.systemInputTaskbar.setChecked(False)
                
        
        # get sound control state
        if common.isControlSound():
            self.systemInputControlSound.setChecked(True)
        else:
            self.systemInputControlSound.setChecked(False)
            
        # get state Oneninedesign power case
        if common.isOneninedesign():
            self.systemInputOneninedesign.setChecked(True)
        else:
            self.systemInputOneninedesign.setChecked(False)
            
        # get state Argop1 power case
        if common.isArgon1():
            self.systemInputArgon1.setChecked(True)
        else:
            self.systemInputArgon1.setChecked(False)
            
        system = common.getSystemTyp()
        if system != "PI4":
            self.systemInputArgon1.setEnabled(False)
            self.systemInputOneninedesign.setEnabled(False)
            
        # get sound cards
        values = common.getSoundCards()
        common.fillComboBoxList(self, values, self.systemInputSound)
        soundcard = common.getCurrentSoundCard()
        stringsplit = soundcard.split(" ")
        cardnumber = stringsplit[0].strip() + " "
        
        for index in range(self.systemInputSound.count()):
            value = self.systemInputSound.itemText(index)
            if value.startswith(cardnumber):
                self.systemInputSound.setCurrentIndex(index)
                break
        
        # set title
        if self.startadmin:
            self.setWindowTitle(common.getRessource("dialogTitle") + " ADMINISTRATOR")
        else:
            self.setWindowTitle(common.getRessource("dialogTitle"))
            
        # start vpn connection
        if autostartvpn:
            parameter = None
            if "parametervpn" in vpnvalues:
                parameter = vpnvalues["parametervpn"]
            systemlogin = None
            if "systemlogin" in vpnvalues:
                systemlogin = vpnvalues["systemlogin"]
            runfile=""
            if "runfile" in vpnvalues:
                runfile = vpnvalues["runfile"]
            if "otp" in vpnvalues:
                otp = vpnvalues["otp"]
            vpn.autostartvpn(self, vpnvalues["program"], vpnvalues["configfile"], parameter, systemlogin, runfile, otp)
        # autostart connections
        self.autostart()
        # read existing connections
        self.fillListWidgets()
        # set network to "not changed"
        self.NetworkChanged = False
        # Start with no configuration mode
        self.ConnectionExtended = False

    def changeText(self, index):
        self.text.setText(self.combo.itemText(index))
        
    def closeEvent(self, event):
        logging.info("close")
        common.systemShutdown(self)
        #event.ignore()

    def fillListWidgets(self):
        common.fillListWidgets(self, self.connectedThreads)

    def wlanPasswordChanged(self):
        self.wlanPasswordChanged = True
        
    def remotePasswordChanged(self):
        self.RemotePasswordChanged = True
    
    def passwordChanged(self):
        self.PasswordChanged = True
        
    def sshPasswordChanged(self):
        self.sshPasswordChanged = True
        
    def rootPasswordChanged(self):
        self.RootPasswordChanged = True

    def userPasswordChanged(self):
        self.UserPasswordChanged = True
        
    def encryptPasswordChanged(self):
        self.EncryptPasswordChanged = True
    
    def networkChanged(self):
        self.NetworkChanged = True
        
    # show wlan password
    def showWlanPassword(self):
        if self.NetworkInputWlanPasswordShow == False:
            self.networkInputWlanPassword.setEchoMode(False)
            self.NetworkInputWlanPasswordShow = True
        else:
            self.networkInputWlanPassword.setEchoMode(QtWidgets.QLineEdit.Password)
            self.NetworkInputWlanPasswordShow = False
    
    # show admin password
    def showPassword(self):
        if self.SystemInputPasswordShow == False:
            self.systemInputPassword.setEchoMode(False)
            self.SystemInputPasswordShow = True
        else:
            self.systemInputPassword.setEchoMode(QtWidgets.QLineEdit.Password)
            self.SystemInputPasswordShow = False

    # show remote password
    def showRemotePassword(self):
        if self.SystemInputRemotePasswordShow == False:
            self.systemInputRemotePassword.setEchoMode(False)
            self.SystemInputRemotePasswordShow = True
        else:
            self.systemInputRemotePassword.setEchoMode(QtWidgets.QLineEdit.Password)
            self.SystemInputRemotePasswordShow = False

    # show ssh password
    def showsshPassword(self):
        if self.SystemInputsshPasswordShow == False:
            self.systemInputsshPassword.setEchoMode(False)
            self.SystemInputsshPasswordShow = True
        else:
            self.systemInputsshPassword.setEchoMode(QtWidgets.QLineEdit.Password)
            self.SystemInputsshPasswordShow = False

    # show root password
    def showRootPassword(self):
        if self.SecurityInputRootPasswordShow == False:
            self.securityInputRootPassword.setEchoMode(False)
            self.SecurityInputRootPasswordShow = True
        else:
            self.securityInputRootPassword.setEchoMode(QtWidgets.QLineEdit.Password)
            self.SecurityInputRootPasswordShow = False

    # show user password
    def showUserPassword(self):
        if self.SecurityInputUserPasswordShow == False:
            self.securityInputUserPassword.setEchoMode(False)
            self.SecurityInputUserPasswordShow = True
        else:
            self.securityInputUserPassword.setEchoMode(QtWidgets.QLineEdit.Password)
            self.SecurityInputUserPasswordShow = False

    def printer(self):
        #common.messageDialog("systemPrinterConfigMessage")
        command = common.getRessource("commandPrinter")
        common.runProgram(command)
        
    def soundCard(self):
        self.systemInputSound.clear()
        values = common.getSoundCards()
        common.fillComboBoxList(self, values, self.systemInputSound)
        soundcard = common.getCurrentSoundCard()
        stringsplit = soundcard.split(" ")
        cardnumber = stringsplit[0].strip() + " "
        
        for index in range(self.systemInputSound.count()):
            value = self.systemInputSound.itemText(index)
            if value.startswith(cardnumber):
                self.systemInputSound.setCurrentIndex(index)
                break
            
    def setMode(self):
        result = common.confirmDialog(self,
                           common.getRessource("systemRebootTitle"),
                           common.getRessource("systemRebootMessage"))
        if result:
            if self.startadmin == False:
                if self.ConnectionExtended:
                    result = True
                    common.setMode("admin")
                else:
                    dialogPassword = dialogpassword.dialogPasswordUI(self.configfile)
                    dialogPassword.exec_()
                    result = dialogPassword.getResult()
                    
                    if result:
                        values = common.readSystem()
                        password = dialogPassword.getPassword()
                        if "adminPassword" in values:
                            inputpassword = common.convertPassword(password)
                            if values["adminPassword"] == inputpassword:
                                common.setMode("admin")
                            else:
                                result = False
                                common.messageDialog("connectExtendedPasswordError")            
                        else:
                            common.setMode("admin")
            else:
                result = True
                common.setMode("user")
                
            if result:
                command = common.getRessource("commandSystemReboot")
                common.runProgram(command)
                command = common.getRessource("commandReboot")
                common.runProgram(command)
            
# Connection
    def connectionExtended(self):
        logging.info("connectionExtended")
        if not self.ConnectionExtended:
            dialogPassword = dialogpassword.dialogPasswordUI(self.configfile)
            dialogPassword.exec_()
            result = dialogPassword.getResult()
            if result is True:
                password = dialogPassword.getPassword()
                values = common.readSystem()
                if "adminPassword" in values:
                    inputpassword = common.convertPassword(password)
                    if values["adminPassword"] == inputpassword:
                        common.enableTabsConfig(self)
                        self.ConnectionExtended = True
                        self.connectButtonExtended.setText(common.getRessource("connectButtonExtendedEnd"))
                    else:
                        common.messageDialog("connectExtendedPasswordError")
                else:
                    common.enableTabsConfig(self)
                    self.ConnectionExtended = True
                    self.connectButtonExtended.setText(common.getRessource("connectButtonExtendedEnd"))
        else:
            common.enableTabsUser(self)
            self.ConnectionExtended = False
            self.connectButtonExtended.setText(common.getRessource("connectButtonExtended"))

        
    def autostart(self):
        logging.info("autostart")
        connections = common.readAllConnections()
        for index in range(len(connections)):
            name = connections[index]
            values = common.readConnection(name)
            if values["autostart"] == "yes":
                self.connectionExec(values)

    def connection(self):
        logging.info("connection")
        item = self.connectList.currentItem()
        if item is not None:
            name = item.text()
            if not name in self.connectedThreads:
                values = common.readConnection(name)
                self.connectionExec(values)
                self.fillListWidgets()

        
    def connectionExec(self, values):
        logging.info("connectionExec")
        # run before
        command = common.getRessource("commandRunBeforeExec")
        common.runProgram(command, False)
        
        repeat = True
        while repeat:
            run = 0
                
            while run < 2:
                connectionname = values["name"]
                command = values["command"]
                systemlogin = values["systemlogin"]
                logging.info("connectionExec " + connectionname)
                
                result = True            
                if systemlogin == "yes":
                    logging.info("loginDialog")
                    dialogLogin = dialoglogin.dialogLoginUI(self.configfile, connectionname)
                    dialogLogin.exec_()
                    result = dialogLogin.getResult()
                    if result is True:
                        # replace parameter <user> and <password> in command string
                        command = command.replace(common.getRessource("commandPlaceholderUser"), dialogLogin.getLogin())
                        password = "'" + dialogLogin.getPassword() + "'"
                        command = command.replace(common.getRessource("commandPlaceholderPassword"), password)
                else:
                    if "login" in values:
                        command = command.replace(common.getRessource("commandPlaceholderUser"), values["login"])
                    if "password" in values:
                        command = command.replace(common.getRessource("commandPlaceholderPassword"), values["password"])
                        
                if result:
                    if "hostalive" in values:
                        if values["hostalive"] == "yes":
                            result = common.isHostAlive(values["address"])
                    else:
                        if "address" in values:
                            result = common.isHostAlive(values["address"])
            
                if result:
                    break
                else:
                    if "alternative" in values:
                        if values["alternative"] != "":                        
                            alternative = values["alternative"]
                            values = []
                            values = common.readConnection(alternative)
                            if not "name" in values:
                                break;
                        else:
                            break
                    else:
                        break
                    
                run = run + 1
                           
            if result:            
                logging.info(command)
                connectionlog = common.getRessource("fileConnectionLog")
                # run connection as thread
                runconnect = connectThread.connectThread(command, connectionname, connectionlog)
                runconnect.threadCancel.connect(self.connectThreadCancel)
                #self.connect(runconnect, QtCore.SIGNAL(
                #             "connectThreadCancel(QString)"), self.connectThreadCancel)
                self.connectedThreads[connectionname] = runconnect
                runconnect.start()
                break
            else:
                #common.messageDialog("connectNoConnection", False)
                repeat = common.cancelDialog("connectNoConnection")            

# called by Button "Cancel connection"
    def connectionCancel(self):
        logging.info("connectionCancel")
        item = self.connectList.currentItem()
        if item is not None:
            name = item.text()
            if name in self.connectedThreads:
                runconnect = self.connectedThreads[name]
                if runconnect is not None:
                    pid = runconnect.getPid()
                    command = common.getRessource("commandKill")
                    command = command + " " + str(pid)
                    common.runProgram(command)

# called by threadConnect when connection end
    def connectThreadCancel(self, name):
        logging.info(name)
        runconnect = self.connectedThreads[name]
        runconnect.terminate()
        # show connection list
        del self.connectedThreads[name]        
        time.sleep(1)
        # repeat connection
        self.repeatConnection(name)
        self.fillListWidgets()

# repeat a cancelled connection
    def repeatConnection(self, name):
        result = common.isRepeat(name)
        logging.info(result)
        if result is True:
            values = common.readConnection(name)
            self.connectionExec(values)

# Configuration
    def configListItem(self):
        logging.info("configList")
        self.configEdit()

    def configNewConnection(self):
        logging.info("newConnection")
        connectiontype = self.configInputConnectionType.currentText()
        if connectiontype == "VNC":
            self.configNewConnectionVNC()
        elif connectiontype == "RDP (xfreerdp)":
            self.configNewConnectionRDP()
        elif connectiontype == "RDP (rdesktop)":
            self.configNewConnectionRDesktop()
        elif connectiontype == "XDMCP":
            self.configNewConnectionxdmcp()
        elif connectiontype == "X2Go":
            self.configNewConnectionx2go()
        elif connectiontype == "ssh":
            self.configNewConnectionssh()
        elif connectiontype == "Browser":
            self.configNewConnectionBrowser()

    def configNewConnectionVNC(self):
        logging.info("newConnectionVNC")
        viewer = common.getVNCViewer()
        if viewer == "tigervnc":
            dialogConnection = dialogtigervnc.dialogTigerVNCUI("", self.configfile)
        if viewer == "tightvnc":
            dialogConnection = dialogtightvnc.dialogTightVNCUI("", self.configfile)
        if viewer == "realvnc":
            dialogConnection = dialogrealvnc.dialogRealVNCUI("", self.configfile)
        dialogConnection.exec_()
        self.fillListWidgets()

    def configNewConnectionRDP(self):
        logging.info("newConnectionRDP")
        dialogConnection = dialogrdp.dialogRDPUI("", self.configfile)
        dialogConnection.exec_()
        self.fillListWidgets()
        
    def configNewConnectionRDesktop(self):
        logging.info("newConnectionRDesktop")
        dialogConnection = dialogrdesktop.dialogRDesktopUI("", self.configfile)
        dialogConnection.exec_()
        self.fillListWidgets()

    def configNewConnectionssh(self):
        logging.info("newConnectionssh")
        dialogConnection = dialogssh.dialogsshUI("", self.configfile)
        dialogConnection.exec_()
        self.fillListWidgets()

    def configNewConnectionxdmcp(self):
        logging.info("newConnectionxdmcp")
        dialogConnection = dialogxdmcp.dialogxdmcpUI("", self.configfile)
        dialogConnection.exec_()
        self.fillListWidgets()

    def configNewConnectionx2go(self):
        logging.info("newConnectionx2go")
        dialogConnection = dialogx2go.dialogx2goUI("", self.configfile)
        dialogConnection.exec_()
        self.fillListWidgets()

    def configNewConnectionBrowser(self):
        logging.info("newConnectionBrowser")
        dialogConnection = dialogbrowser.dialogbrowserUI("", self.configfile)
        dialogConnection.exec_()
        self.fillListWidgets()
    
    def configEdit(self):
        logging.info("configEdit")
        item = self.configList.currentItem()
        if item is not None:
            name = item.text()
            connection = common.readConnection(name)
            if connection["typ"] == "vnc":
                viewer = common.getVNCViewer()
                if viewer == "tigervnc":
                    dialogConnection = dialogtigervnc.dialogTigerVNCUI(name, self.configfile)
                if viewer == "tightvnc":
                    dialogConnection = dialogtightvnc.dialogTightVNCUI(name, self.configfile)
                if viewer == "realvnc":
                    dialogConnection = dialogrealvnc.dialogRealVNCUI(name, self.configfile)
                dialogConnection.exec_()
            if connection["typ"] == "ssh":
                dialogConnection = dialogssh.dialogsshUI(name, self.configfile)
                dialogConnection.exec_()
            if connection["typ"] == "xdmcp":
                dialogConnection = dialogxdmcp.dialogxdmcpUI(name, self.configfile)
                dialogConnection.exec_()
            if connection["typ"] == "x2go":
                dialogConnection = dialogx2go.dialogx2goUI(name, self.configfile)
                dialogConnection.exec_()
            if connection["typ"] == "rdp":
                dialogConnection = dialogrdp.dialogRDPUI(name, self.configfile)
                dialogConnection.exec_()
            if connection["typ"] == "rdesktop":
                dialogConnection = dialogrdesktop.dialogRDesktopUI(name, self.configfile)
                dialogConnection.exec_()
            if connection["typ"] == "browser":
                dialogConnection = dialogbrowser.dialogbrowserUI(name, self.configfile)
                dialogConnection.exec_()
        self.fillListWidgets()

    def configDelete(self):
        logging.info("configDelete")
        item = self.configList.currentItem()
        if item is not None:
            result = common.confirmDialog(self,
                                          common.getRessource("configDeleteDialogTitle"),
                                          common.getRessource("configDeleteDialogMessage"))
            if result:
                common.deleteConnection(item.text())
                self.fillListWidgets()

# Network
# -------
    def networkType(self):
        if self.networkInputDHCP.isChecked():
            self.networkInputAddress.setEnabled(False)
            self.networkInputGateway.setEnabled(False)
            self.networkInputSubnetmask.setEnabled(False)
            logging.info("networkDHCP")
        else:
            self.networkInputStaticIP.setChecked(True)
            self.networkInputAddress.setEnabled(True)
            self.networkInputGateway.setEnabled(True)
            self.networkInputSubnetmask.setEnabled(True)
            logging.info("networkStaticIP")

# System
# ------
    def systemTerminal(self):
        logging.info("systemTerminal")
        common.systemTerminal()

    def systemCaseArgon1(self):
        logging.info("systemCaseTypeArgon1")
        if self.systemInputArgon1.isChecked():
            self.systemInputOneninedesign.setChecked(False)
            
    def systemCaseOneninedesign(self):
        logging.info("systemCaseTypeOneninedesign")    
        if self.systemInputOneninedesign.isChecked():
            self.systemInputArgon1.setChecked(False)

    def searchSSID(self):
        logging.info("searchSSID")
        values = common.getWlan()
        self.networkInputWlanSSID.clear()
        common.fillComboBoxList(self, values, self.networkInputWlanSSID)
        
    def systemReboot(self):
        if self.EncryptPasswordChanged or self.systemInputEncrypt.isChecked():
            common.setEncryptPassword(self.systemInputPasswordEncrypt.text())
        common.systemReboot(self)

    def systemShutdown(self):
        if self.EncryptPasswordChanged or self.systemInputEncrypt.isChecked():
            common.setEncryptPassword(self.systemInputPasswordEncrypt.text())
        common.systemShutdown(self)
     
    def connectionLog(self):
        common.connectionLog(self)
        
    def backgroundImageGetFile(self):
        logging.info("backgroundImageGetFile")
        dialog = QtWidgets.QFileDialog()
        dialog.setFileMode(QtWidgets.QFileDialog.AnyFile)
        dialog.setDirectory(common.getRessource("backgroundImageDirectory"))
        filename = ""
        if dialog.exec_():
            filenames = dialog.selectedFiles()
            filename = filenames[0]
        self.systemInputBackgroundImage.setText(filename)
        
# vpn
# ---
    def vpnClearGetFile(self):
        logging.info("vpnClearGetFile")
        self.vpnInputFile.setText("")
        
    def vpnGetFile(self):
        logging.info("vpnGetFile")
        vpn.vpnGetFile(self)
        
    def vpnRunFile(self):
        logging.info("vpnRunFile")
        vpn.vpnRunFile(self)
        
    def vpnTransfer(self):
        logging.info("vpnTransfer")
        vpn.vpnTransfer(self)        
        # reboot system
        result = common.confirmDialog(self,
                                      common.getRessource("systemShutdownTitle"),
                                      common.getRessource("systemAssumeMessage"))
        if result:
            common.runProgram(common.getRessource("commandReboot"))
        
    def vpnConnect(self):
        logging.info("vpnConnection")
        vpn.vpnConnect(self)
            
    def vpnCancel(self):
        logging.info("vpnCancel")
        vpn.vpnCancel(self)
            
    def vpnCondition(self): 
        logging.info("vpnCondition")
        vpn.vpnCondition(self)
        
    def vpnAdditionalFiles(self): 
        logging.info("vpnAdditionalFiles")
        vpn.vpnAdditionalFiles(self)

    def vpnDeleteFile(self): 
        logging.info("vpnDeleteFile")
        vpn.vpnDeleteFile(self)

# save and restart
# ----------------
    def save(self):
        logging.info("save and restart")
        values = {}
        # admin password
        password = self.systemInputPassword.text()
        if self.PasswordChanged and password != "":
            values["adminPassword"] = common.convertPassword(password)
        
        # wlan edited?
        if self.networkInputSave.isChecked() == False:
            if self.NetworkChanged:
                if self.networkInputWlanSSID.currentText() != "":
                    if self.networkInputWlanPassword.text() == "":
                        common.messageDialog("networkWlanError")
                        return
                if self.networkInputWlanPassword.text() != "":
                    if self.networkInputWlanSSID.currentText() == "":
                        common.messageDialog("networkWlanError")
                        return
                wlanpassword = self.networkInputWlanPassword.text()
                if wlanpassword != "" and len(wlanpassword) < 8:
                    common.messageDialog("networkWlanPasswordError")
                    return

        # action time is string, convert to datetime or hour:minute
        value = self.timeInputActionTime.text()
        if value != "":
            # hour:minute
            if value.startswith("+"):
                value = value.replace("+", "")
                try:
                    time = datetime.strptime(value, "%H:%M")
                except ValueError as error:
                    common.messageDialog("timeActionError")
                    return
            else:
                try:
                    time = datetime.strptime(value, "%Y-%m-%d %H:%M")
                except ValueError as error:
                    common.messageDialog("timeActionError")
                    return
            
        # network
        if self.networkInputSave.isChecked() == False:
            if self.NetworkChanged:
                networkvalues = {}
                networkvalues["connection"] = self.networkInputInterface.currentText()
                networkvalues["address"] = self.networkInputAddress.text()
                networkvalues["subnetmask"] = self.networkInputSubnetmask.text()
                networkvalues["gateway"] = self.networkInputGateway.text()
                networkvalues["dns"] = self.networkInputDNS.text()
                networkvalues["ssid"] = self.networkInputWlanSSID.currentText()
                networkvalues["password"] = self.networkInputWlanPassword.text()
                
                if self.networkInputDHCP.isChecked():
                    common.networkWriteDHCP(networkvalues)
                else:
                    result = common.networkWriteStaticIP(networkvalues)
                    if not result:
                        return
           
        # vpn
        vpn.vpnTransfer(self)
        
        if self.systemInputRemoteVNC.isChecked():
            if self.RemotePasswordChanged:
                common.remoteVNC(True, self.systemInputRemotePassword.text())
        else:
            common.remoteVNC(False, None)
            
        if self.systemInputSSH.isChecked():
            if self.sshPasswordChanged:
                common.remoteSSH(True, self.systemInputsshPassword.text())
        else:
            common.remoteSSH(False, None)
        
        values["resolution"] = self.systemInputResolution.currentText()
        values["resolution2"] = self.systemInputResolution2.currentText()
        
        common.setOptimizeGraphic(self.systemInputOptimizeGraphic.isChecked())
        
        orientation = "left"
        if self.systemInputMultiMonitorLeft.isChecked():
            orientation = "left"
        if self.systemInputMultiMonitorRight.isChecked():
            orientation = "right"
        if self.systemInputMultiMonitorAbove.isChecked():
            orientation = "above"
        if self.systemInputMultiMonitorBelow.isChecked():
            orientation = "below"
            
        common.setMultiMonitor(True, orientation)
        
        common.setBackgroundColor(self.systemInputBackgroundColor.text())
        common.setBackgroundImage(self.systemInputBackgroundImage.text())
        
        values["language"] = self.systemInputLanguage.currentText()
        values["keyboardLayout"] = self.systemInputKeyboardLayout.currentText()
        
        values["startMinimized"] = "no"
        if self.systemInputMinimize.isChecked():
            values["startMinimized"] = "yes"
        
        values["networkSave"] = "yes"
        if self.networkInputSave.isChecked():
            values["networkSave"] = "no"
            
        values["simplemenu"] = "no"
        if self.securityInputMenu.isChecked():
            values["simplemenu"] = "yes"
            
        values["virtualconsole"] = "yes"
        if self.securityInputVTerminal.isChecked():
            values["virtualconsole"] = "no"
        
        values["numerickeypad"] = "no"
        if self.systemInputNumericKeypad.isChecked():
            values["numerickeypad"] = "yes"
        
        values["multimonitor"] = "no"
        if self.systemInputMultiMonitor.isChecked():
            values["multimonitor"] = "yes"
            
        # write system file
        common.writeSystem(values)
        
        common.writeKeyboardLayout(values["keyboardLayout"])
        # write system file        
        common.deleteConfigScript()
        
        if self.systemInputScreensaver.isChecked():
            common.setScreensaver("on")
        else:
            common.setScreensaver("off")
            
        if self.systemInputMonitorStandby.isChecked():
            common.setMonitorStandby("on")
        else:
            common.setMonitorStandby("off")
            
        if self.systemInputTaskbar.isChecked():
            common.setTaskbar("on")
        else:
            common.setTaskbar("off")
            
        if self.systemInputControlSound.isChecked():
            common.setControlSound("on")
        else:
            common.setControlSound("off")
                
        common.setResolution(self.systemInputResolution.currentText())
        
        if self.systemInputUSBAutomount.isChecked():
            common.setUSBAutomount("on")
        else:
            common.setUSBAutomount("off")
            
        if self.systemInputOneninedesign.isChecked():
            common.setOneninedesign("on")
        else:
            common.setOneninedesign("off")
            
        if self.systemInputArgon1.isChecked():
            common.setArgon1("on")
        else:
            common.setArgon1("off")
                
        common.setSoundCard(self.systemInputSound.currentText())

        if self.RootPasswordChanged:
            common.setRootPassword(self.securityInputRootPassword.text())
        
        if self.UserPasswordChanged:
            common.setUserPassword(self.securityInputUserPassword.text())
            
        if self.EncryptPasswordChanged or self.systemInputEncrypt.isChecked():
            if self.systemInputPasswordEncrypt.text() == self.systemInputPasswordRepeatEncrypt.text():
                common.setEncryptPassword(self.systemInputPasswordEncrypt.text())
            else:
                common.messageDialog("securityEncryptPasswordError")
                return
                
        common.setSystemname(self.systemInputSystemname.text())
        
        # write time
        values = {}
        values["save"] = "yes"
        if self.timeInputSave.isChecked():
            values["save"] = "no"
        values["automatic"] = "no"
        if self.timeInputAuto.isChecked():
            values["automatic"] = "yes"
        values["ntp"] = self.timeInputNTP.text()
        values["zone"] = self.timeInputTimezone.currentText()
        values["repeat"] = "no"
        if self.timeInputActionRepeat.isChecked():
            values["repeat"] = "yes"
        values["time"] = self.timeInputTime.text()
        values["execute"] = "no"
        if self.timeInputAction.isChecked():
            values["execute"] = "yes"
        values["action"] = self.timeInputActionAction.currentText()
        values["execat"] = self.timeInputActionTime.text()
        common.writeTime(values)
        
        # write time temp. file for save and reboot
        values = {}
        values["show"] = "no"
        if self.timeInputView.isChecked():
            values["show"] = "yes"
        common.writeTimeTemp(values)
        
        # reboot system
        result = common.confirmDialog(self,
                                      common.getRessource("systemShutdownTitle"),
                                      common.getRessource("systemAssumeMessage"))
        if result:
            command = common.getRessource("commandSystemConfig")
            # run configuration in thread
            runconfig = connectThread.connectThread(command, "config", "/tmp/config.log")
            runconfig.finished.connect(self.configFinished)
            runconfig.threadCancel.connect(self.configThreadCancel)
            runconfig.start()
            # show message dialgo during configuration
            dialogConfiguration = configurationdialog.dialogConfigurationUI(self.configfile, common.getRessource("configurationDialogMessage"))
            dialogConfiguration.exec_()

    # reboot system, when configuration is finished
    def configFinished(self):
        command = common.getRessource("commandReboot")
        common.runProgram(command)
    
    def configThreadCancel(self):
        pass
    
def main():
    configfile = ""
    if len(sys.argv) > 1:
        args = sys.argv
        configfile = args[1]

    app = QtWidgets.QApplication(sys.argv)
    form = thinClientUI(configfile)
    form.show()
    app.exec_()

if __name__ == "__main__":
    main()
