#!/bin/bash
# create raspberry os image on sd card
# $1 = device name (i.e. /dev/sdb)
# $2 = image name
#
# delete all partitions on sd card
echo "* clear sd card"
parted $1 --script -- rm 1 2> /dev/null
parted $1 --script -- rm 2 2> /dev/null
parted $1 --script -- rm 3 2> /dev/null
parted $1 --script -- rm 4 2> /dev/null
parted $1 --script -- rm 5 2> /dev/null

# copy image to sd card
echo "* copy image to sd card"
dd bs=4M if=$2 of=$1 status=progress
sync

# delete resize statement from boot sequence
echo "* remove resize statement"
mount $1\1 /mnt
sed -i "s/init.*$/ /" /mnt/cmdline.txt
umount /mnt

# compute size of root partition
FREE=$(parted /dev/sdb unit MB print free | grep "^  " |tail -1 | awk '{print $3}' | cut -f1 -d"M")
DATA=$(($FREE/8))
#ROOT=$(($FREE-$DATA))
ROOT=$(parted /dev/sdb unit MB print free | grep "^ 2" |grep "^ 2" | awk '{print $4}' | cut -f1 -d"M")
ROOT=$(($FREE-$DATA+$ROOT))
echo "* size root partition:" $ROOT

# resize root partition
echo "* resize root partition"
parted $1 --script -- resizepart 2 $ROOT\MB
resize2fs $1\2

# create data partition
#FREE=$(parted /dev/sdb unit MB print free | grep "^  " |tail -1 | awk '{print $1} {print $2}')
START=$(parted /dev/sdb unit MB print free | grep "^  " |tail -1 | awk '{print $1}')
END=$(parted /dev/sdb unit MB print free | grep "^  " |tail -1 | awk '{print $2}')
echo "* size data partition:" $START $END
parted $1 --script -- mkpart primary ext3 $START $END > /dev/null
sync
mkfs.ext3 -F $1\3
sync
echo "* sd card ready"
exit 0
