#!/bin/bash
# Install xtc 
#
SOURCE=`dirname $0`
rm $SOURCE/install.err 2> /dev/null
DIR=/home/thinclient
DESTINATION=/opt/thinclient
VAR=/var/thinclient
DATABASE=/data
DATA=$DATABASE"/user"
DATAENC=$DATABASE"/enc"
echo "* start installation"
MODEL="22.04"
OS="U"
if (cat /etc/issue | grep -i "RASPBIAN") || (cat /etc/issue | grep -i "DEBIAN")
then
  OS="R"
  # detect pi hardware 
  MODEL=$(cat /proc/cpuinfo |grep "Revision" | cut -d: -f2 | sed 's/^[ \t]*//')
  if [ "$MODEL" = "a22082" ] || [ "$MODEL" = "a220a0" ] || [ "$MODEL" = "a32082" ] || [ "$MODEL" = "a52082" ] || [ "$MODEL" = "a22083" ] || [ "$MODEL" = "a02100" ] || [ "$MODEL" = "a020d3" ] || [ "$MODEL" = "a2082" ] || [ "$MODEL" = "a20a0" ]
  then
    MODEL="PI3"
  else
    MODEL="PI4"
  fi
else
  MODEL="22"
  if (cat /etc/issue | grep -i "23.10")
  then
    MODEL="23"
  fi
fi

echo "System:" $OS
echo "Release:" $MODEL

echo "1. install packages"
apt-get -y update > $SOURCE/install.out 2> $SOURCE/install.err
apt-get -y upgrade > $SOURCE/install.out 2>> $SOURCE/install.err
if [ "$OS" = "R" ]
then
   echo "   - install raspberrypi-kernel-headers"
   apt-get -y install raspberrypi-kernel-headers >> $SOURCE/install.out 2>> $SOURCE/install.err
fi
echo "   - install ssh"
apt-get -y install ssh >> $SOURCE/install.out 2>> $SOURCE/install.err
echo "   - install sshpass"
apt-get -y install sshpass >> $SOURCE/install.out 2>> $SOURCE/install.err
echo "   - install nfs-common"
apt-get -y install nfs-common >> $SOURCE/install.out 2>> $SOURCE/install.err
# get wlan ssid
echo "   - install wireless-tools"
apt-get -y install wireless-tools >> $SOURCE/install.out 2>> $SOURCE/install.err
echo "   - install fbi"
apt-get -y install fbi >> $SOURCE/install.out 2>> $SOURCE/install.err
if [ "$OS" = "R" ]
then
  echo "   - install xorg"
  apt-get -y install xorg >> $SOURCE/install.out 2>> $SOURCE/install.err
  echo "   - install flatpak"
  apt-get -y install flatpak >> $SOURCE/install.out 2>> $SOURCE/install.err
else
  echo "   - install xserver-xorg-core"
  apt-get -y install xserver-xorg-core >> $SOURCE/install.out 2>> $SOURCE/install.err
fi
echo "   - install x11vnc"
apt-get -y install x11vnc >> $SOURCE/install.out 2>> $SOURCE/install.err
echo "   - install openbox"
apt-get -y install openbox >> $SOURCE/install.out 2>> $SOURCE/install.err 
if [ "$OS" = "U" ]
then
  echo "   - install xinit"
  apt-get -y install xinit >> $SOURCE/install.out 2>> $SOURCE/install.err
  echo "   - install xterm"
  apt-get -y install xterm >> $SOURCE/install.out 2>> $SOURCE/install.err
fi
# install firefox and chromium
echo "   - install browser"
if [ "$OS" = "R" ]
then
  apt-get -y install firefox-esr firefox-esr-l10n-de >> $SOURCE/install.out 2>> $SOURCE/install.err
  ln -s /usr/bin/firefox-esr /usr/local/bin/firefox >> $SOURCE/install.out 2>> $SOURCE/install.err
  apt-get -y install chromium-browser chromium-browser-l10n >> $SOURCE/install.out 2>> $SOURCE/install.err
  ln -s /usr/bin/chromium-browser /usr/local/bin/chromium >> $SOURCE/install.out 2>> $SOURCE/install.err
else
  # firefox
  cp ubuntu_mozillateam /etc/apt/preferences.d/mozillateam >> $SOURCE/install.out 2>> $SOURCE/install.err
  add-apt-repository -y ppa:mozillateam/ppa >> $SOURCE/install.out 2>> $SOURCE/install.err
  apt-get -y update >> $SOURCE/install.out 2>> $SOURCE/install.err
  apt-get -y install firefox-esr firefox-esr-locale-de >> $SOURCE/install.out 2>> $SOURCE/install.err
  ln -s /usr/bin/firefox-esr /usr/local/bin/firefox >> $SOURCE/install.out 2>> $SOURCE/install.err
  # chromium
  if [ "$MODEL" != "23" ]
  then
    add-apt-repository -y ppa:savoury1/chromium >> $SOURCE/install.out
  fi
  apt-get -y update >> $SOURCE/install.out 2>> $SOURCE/install.err
  apt-get install -y chromium-browser >> $SOURCE/install.out 2>> $SOURCE/install.err
  ln -s /usr/bin/chromium-browser /usr/local/bin/chromium >> $SOURCE/install.out 2>> $SOURCE/install.err
fi
echo "   - install tint2"
apt-get -y install tint2 >> $SOURCE/install.out 2>> $SOURCE/install.err
echo "   - install wmctrl"
apt-get -y install wmctrl >> $SOURCE/install.out 2>> $SOURCE/install.err
#echo "   - install qt5-default"
#apt-get -y install qt5-default >> $SOURCE/install.out 2>> $SOURCE/install.err
echo "   - install qtbase5-dev"
apt-get -y install qtbase5-dev >> $SOURCE/install.out 2>> $SOURCE/install.err
echo "   - install qt5-qmake"
apt-get -y install qt5-qmake >> $SOURCE/install.out 2>> $SOURCE/install.err
echo "   - install python3-pyqt5"
apt-get -y install python3-pyqt5 >> $SOURCE/install.out 2>> $SOURCE/install.err
# python3 ntp time library
echo "   - install python3-ntplib"
apt-get -y install python3-ntplib >> $SOURCE/install.out 2>> $SOURCE/install.err
echo "   - install python3-yaml"
apt-get -y install python3-yaml >> $SOURCE/install.out 2>> $SOURCE/install.err
# python3 time zone library
echo "   - install pytho3-tz"
apt-get -y install python3-tz >> $SOURCE/install.out 2>> $SOURCE/install.err
# python3 pip
echo "   - install python3-pip"
apt-get -y install python3-pip >> $SOURCE/install.out 2>> $SOURCE/install.err
# python3 smbus (argon1 case)
echo "   - install python3-smbus"
apt-get -y install python3-smbus >> $SOURCE/install.out 2>> $SOURCE/install.err
# python3 pycryptodome
echo "   - install python3-pycryptodome"
apt-get -y install python3-pycryptodome >> $SOURCE/install.out 2>> $SOURCE/install.err
# python3 network
echo "   - install python3-netifaces"
apt-get -y install python3-netifaces >> $SOURCE/install.out 2>> $SOURCE/install.err
echo "   - install xbase-clients"
apt-get -y install xbase-clients >> $SOURCE/install.out 2>> $SOURCE/install.err
echo "   - install tightvncserver"
apt-get -y install tightvncserver >> $SOURCE/install.out 2>> $SOURCE/install.err
#echo "   - install xtightvncviewer"
#apt-get -y install xtightvncviewer >> $SOURCE/install.out
echo "   - install tigervnc-viewer"
apt-get -y install tigervnc-viewer >> $SOURCE/install.out 2>> $SOURCE/install.err
echo "   - install freerdp2-x11"
apt-get -y install freerdp2-x11 >> $SOURCE/install.out 2>> $SOURCE/install.err
echo "   - install rdesktop"
apt-get -y install rdesktop >> $SOURCE/install.out 2>> $SOURCE/install.err
echo "   - install xserver-xephyr"
apt-get -y install xserver-xephyr >> $SOURCE/install.out 2>> $SOURCE/install.err
echo "   - install pyhoca-cli"
apt-get -y install pyhoca-cli >> $SOURCE/install.out 2>> $SOURCE/install.err
echo "   - install openvpn"
apt-get -y install openvpn >> $SOURCE/install.out 2>> $SOURCE/install.err
echo "   - install wireguard"
apt-get -y install wireguard >> $SOURCE/install.out 2>> $SOURCE/install.err
if [ "$OS" = "U" ]
then
  apt-get -y install resolvconf >> $SOURCE/install.out 2>> $SOURCE/install.err
fi
# used by usbmount
echo "   - install lockfile-progs"
apt-get -y install lockfile-progs >> $SOURCE/install.out 2>> $SOURCE/install.err
# application to config dual monitor 
echo "   - install arandr"
apt-get -y install arandr >> $SOURCE/install.out 2>> $SOURCE/install.err
echo "   - install numlockx"
apt-get -y install numlockx >> $SOURCE/install.out 2>> $SOURCE/install.err
# install alsa for ubuntu
if [ "$OS" = "U" ]
then
  echo "   - install alsa (Ubuntu only)"
  apt-get -y install alsa-base >> $SOURCE/install.out 2>> $SOURCE/install.err
  apt-get -y install alsa-utils >> $SOURCE/install.out 2>> $SOURCE/install.err
  echo "   - install pulseaudio (Ubuntu only)"
  apt-get -y install pulseaudio >> $SOURCE/install.out 2>> $SOURCE/install.err
  apt-get -y install pavucontrol >> $SOURCE/install.out 2>> $SOURCE/install.err
else
  echo "   - install qasmixer (Raspberry Pi only)"
  apt-get -y install qasmixer >> $SOURCE/install.out 2>> $SOURCE/install.err
fi

# create desktop icons
echo "   - install idesk" 
apt-get -y install idesk >> $SOURCE/install.out 2>> $SOURCE/install.err
echo "   - install usbmount"
if [ "$OS" = "R" ]
then
  # original usbmount is defect
  dpkg -i $SOURCE/usbmount_0.0.24_all.deb >> $SOURCE/install.out 2>> $SOURCE/install.err
else
  dpkg -i $SOURCE/usbmount_0.0.24_all.deb >> $SOURCE/install.out 2>> $SOURCE/install.err
  #apt-get -y install usbmount >> $SOURCE/install.out 2>> $SOURCE/install.err
fi

# Raspberry Pi: delete pulseaudio while problems with alsa sound

if [ "$OS" = "R" ]
then
  apt-get -y remove pulseaudio >> $SOURCE/install.out 2>> $SOURCE/install.err
fi

# install cups
echo "   - install cups" 
apt-get -y install cups hplip printer-driver-gutenprint >> $SOURCE/install.out 2>> $SOURCE/install.err
# check webcam
echo "   - install guvcview" 
apt-get -y install guvcview >> $SOURCE/install.out 2>> $SOURCE/install.err
# libraries xfreerdp3
if [ "$OS" = "U" ]
then
  apt-get -y install libcjson1 libfuse2 >> $SOURCE/install.out 2>> $SOURCE/install.err
fi
echo "   - install feh"
apt-get -y install feh >> $SOURCE/install.out 2>> $SOURCE/install.err
# install splashscreen
echo "   - install plymouth"
apt-get install -y plymouth plymouth-themes >> $SOURCE/install.out 2>> $SOURCE/install.err
# install gocryptfs
echo "   - install gocryptfs"
apt-get install -y libpam-mount gocryptfs

apt-get -y autoremove >> $SOURCE/install.out 2>> $SOURCE/install.err

echo "2. config ssh"
# only user thinclientssh and system is allowed
if (grep "^AllowUsers" /etc/ssh/sshd_config)
then
  sed -e 's/^AllowUsers.*$/AllowUsers thinclientssh system/g' -i /etc/ssh/sshd_config
else
  echo "AllowUsers thinclientssh system" >> /etc/ssh/sshd_config
fi
# disable ssh
systemctl disable ssh > /dev/null 2>&1

echo "3. add user"
adduser --disabled-password --gecos "" thinclient 2>> $SOURCE/install.err
echo thinclient:thinclient | chpasswd 2>> $SOURCE/install.err
# ssh user
adduser --disabled-password --gecos "" thinclientssh 2>> $SOURCE/install.err
echo thinclientssh:thinclient | chpasswd 2>> $SOURCE/install.err
# give ssh user same uid/gid as thinclient user
USERID=$(id -u thinclient)
usermod -a -G users thinclient 2>> $SOURCE/install.err
usermod -a -G video thinclient 2>> $SOURCE/install.err
usermod -a -G lpadmin thinclient 2>> $SOURCE/install.err
usermod -o -u $USERID thinclientssh
GROUPID=$(id -g thinclient)
groupmod -o -g $GROUPID thinclientssh
mkdir /home/thinclient/.idesktop
chown thinclient:thinclient /home/thinclient/.idesktop
cp $SOURCE/ideskrc /home/thinclient/.ideskrc
chown thinclient:thinclient /home/thinclient/.ideskrc

echo "4. create restore files"
mkdir /thinclient_restore 2>> $SOURCE/install.err
cp /etc/fstab /thinclient_restore 2>> $SOURCE/install.err
if [ "$OS" = "U" ]
then
  cp /etc/netplan/* /thinclient_restore 2>> $SOURCE/install.err
  cp /etc/apt/apt.conf.d/20auto-upgrades /thinclient_restore 2>> $SOURCE/install.err
else
  cp /etc/resolv.conf /thinclient_restore
  cp /etc/wpa_supplicant/wpa_supplicant.conf /thinclient_restore
  cp /lib/systemd/system/wpa_supplican.service /thinclient_restore

fi
cp /lib/systemd/system/getty@.service /thinclient_restore 2>> $SOURCE/install.err
cp /etc/xdg/openbox/menu.xml /thinclient_restore 2>> $SOURCE/install.err
cp /etc/default/keyboard /thinclient_restore 2>> $SOURCE/install.err
cp $DIR/.bashrc /thinclient_restore 2>> $SOURCE/install.err
cp /etc/xdg/openbox/rc.xml /thinclient_restore 2>> $SOURCE/install.err
cp /etc/usbmount/usbmount.conf /thinclient_restore 2>> $SOURCE/install.err
cp /etc/usbmount/usbmount.conf /etc/usbmount/usbmount.conf.orig 2>> $SOURCE/install.err
cp /etc/xdg/user-dirs.defaults /thinclient_restore 2>> $SOURCE/install.err

echo "5. config sudo"
cat /etc/sudoers $SOURCE/sudoers.txt > /tmp/sudoers 2>> $SOURCE/install.err 
mv /tmp/sudoers /etc/sudoers 2>> $SOURCE/install.err

echo "6. autologin"
sed -e 's/^ExecStart=.*$/ExecStart=-\/sbin\/agetty --noclear --autologin thinclient %I $TERM/g' -i /lib/systemd/system/getty@.service 2>> $SOURCE/install.err

echo "7. config openbox"
cp $SOURCE/config/menu.xml /etc/xdg/openbox 2>> $SOURCE/install.err
mkdir $DIR/.config 2>> $SOURCE/install.err
mkdir $DIR/.config/openbox 2>> $SOURCE/install.err
cp $SOURCE/autostart $DIR/.config/openbox 2>> $SOURCE/install.err
cp $SOURCE/rc.xml /etc/xdg/openbox 2>> $SOURCE/install.err
# delete idesk default file, because this file is copied to the desktop
rm /usr/share/idesk/default.lnk
# do not create home folders
cp $SOURCE/user-dirs.defaults /etc/xdg

echo "8. config tint2"
mkdir $DIR/.config/tint2 2>> $SOURCE/install.err
if [ "$OS" = "R" ]
then
  cp $SOURCE/tint2rc $DIR/.config/tint2 2>> $SOURCE/install.err
else
  cp $SOURCE/tint2rc_ubuntu $DIR/.config/tint2/tint2rc 2>> $SOURCE/install.err
fi
chown -R thinclient $DIR/.config 2>> $SOURCE/install.err
chgrp -R thinclient $DIR/.config 2>> $SOURCE/install.err

echo "9. config user thinclient"
cat $DIR/.bashrc $SOURCE/bashrc.txt > /tmp/bashrc 2>> $SOURCE/install.err 
mv /tmp/bashrc $DIR/.bashrc 2>> $SOURCE/install.err
# needed for xfreerdp 3
if [ "$OS" = "R" ]
then
  echo "XDG_SESSION_TYPE=x11" >> $DIR/.bashrc 2>> $SOURCE/install.err
fi
chown thinclient $DIR/.bashrc
chgrp thinclient $DIR/.bashrc

echo "10. install thinclient"
mkdir $DESTINATION 2>> $SOURCE/install.err  
cp -a $SOURCE/bin $DESTINATION 2>> $SOURCE/install.err
cp -a $SOURCE/config $DESTINATION 2>> $SOURCE/install.err  
cp -a $SOURCE/script $DESTINATION 2>> $SOURCE/install.err
cp -a $SOURCE/desktop $DESTINATION 2>> $SOURCE/install.err
if [ "$OS" = "U" ]
then
  cp -a $SOURCE/script/ubuntu/tightvncpasswd $DESTINATION/script 2>> $SOURCE/install.err
else
  cp /usr/bin/tightvncpasswd $DESTINATION/script 2>> $SOURCE/install.err
fi

# set links for terminal commands
ln -s $DESTINATION/script/thinclient.sh /usr/bin/thinclient
ln -s $DESTINATION/script/system.sh /usr/bin/thinclient-config
ln -s $DESTINATION/script/restartxtc.sh /usr/bin/restart
ln -s $DESTINATION/script/timeaction.sh /usr/bin/timeaction
chown -R root:root $DESTINATION

# Since Ubuntu 23.10 network config file is only readable as root
if [ "$OS" = "U" ]
then
  chmod u=rw,g=r,o=r /etc/netplan/*
fi

echo "11. create var directory"
mkdir $VAR 2>> $SOURCE/install.err
mkdir $VAR/home 2>> $SOURCE/install.err

echo "12. save home directory"
cp -a /home/thinclient $VAR/home
chown -R thinclient $VAR 2>> $SOURCE/install.err
chgrp -R thinclient $VAR 2>> $SOURCE/install.err

echo "13. install start script"
cp $SOURCE/thinclient_start.service /etc/systemd/system 2>> $SOURCE/install.err
chown root:root /etc/systemd/system/thinclient_start.service
chmod u=rwx,g=rx,o=rx /etc/systemd/system/thinclient_start.service
systemctl enable thinclient_start.service

#echo "14. install splashscreen"
#cp $SOURCE/splashscreen.png $VAR
#cp $SOURCE/splashscreen.service /etc/systemd/system
#systemctl enable splashscreen

echo "15. mount data directory"
if [ "$OS" = "R" ]
then
  mkdir -p $DATABASE 2>> /dev/null 2>> $SOURCE/install.err
  cat /etc/fstab $SOURCE/fstab_data > /tmp/fstab
  mv /tmp/fstab /etc/fstab
  mount -a
fi

echo "16. create data directory"
mkdir -p $DATA 2>> /dev/null 2>> $SOURCE/install.err
mkdir -p $DATAENC 2>> /dev/null 2>> $SOURCE/install.err


echo "17. config data directory"
#chmod a=rwx $DATA 2>> /dev/null
chown thinclient:thinclient $DATAENC 2>> $SOURCE/install.err
chown thinclient:thinclient $DATA 2>> $SOURCE/install.err
mkdir $DATA/thinclient 2>> $SOURCE/install.err
chown thinclient $DATA/thinclient 2>> $SOURCE/install.err
chgrp thinclient $DATA/thinclient 2>> $SOURCE/install.err
mkdir $DATA/thinclient/config 2>> $SOURCE/install.err
chown thinclient $DATA/thinclient/config 2>> $SOURCE/install.err
chgrp thinclient $DATA/thinclient/config 2>> $SOURCE/install.err
mkdir $DATA/vpn 2>> $SOURCE/install.err
chown thinclient $DATA/vpn 2>> $SOURCE/install.err
chgrp thinclient $DATA/vpn 2>> $SOURCE/install.err
#mkdir $DATA/usb 2>> $SOURCE/install.err
#chown thinclient $DATA/usb 2>> $SOURCE/install.err
#chgrp thinclient $DATA/usb 2>> $SOURCE/install.err
ln -s /media/usb $DATA/usb 2>> $SOURCE/install.err

echo "18. config browser"
mkdir $DATA/thinclient/mozilla
chown thinclient:thinclient $DATA/thinclient/mozilla
ln -s $DATA/thinclient/mozilla /var/thinclient/home/thinclient/.mozilla
chown thinclient:thinclient /var/thinclient/home/thinclient/.mozilla
mkdir $DATA/thinclient/chromium
chown thinclient:thinclient $DATA/thinclient/chromium
ln -s $DATA/thinclient/chromium /var/thinclient/home/thinclient/.config/chromium
chown thinclient:thinclient /var/thinclient/home/thinclient/.config/chromium

echo "19. disable NetworkManager (Raspberry Pi only)"
if [ "$OS" = "R" ]
then
  systemctl stop NetworkManager >> $SOURCE/install.out 2>> $SOURCE/install.err
  systemctl disable NetworkManager >> $SOURCE/install.out 2>> $SOURCE/install.err
  apt-get install -y dhcpcd5 >> $SOURCE/install.out 2>> $SOURCE/install.err
  cp /etc/dhcpcd.conf /thinclient_restore
  sed -i "s/^ExecStart=.*$/& -c\/etc\/wpa_supplicant\/wpa_supplicant.conf -iwlan0/" /lib/systemd/system/wpa_supplican.service
fi

#echo "20. config /boot read only fs"
#if [ "$OS" = "R" ]
#then
  #sed '/\/ /s/defaults/ro,defaults/g' -i /etc/fstab
  #sed '/\/boot/s/defaults/ro,defaults/g' -i /etc/fstab
  #cat /etc/fstab $SOURCE/fstab_ro > /tmp/fstab
  #mv /tmp/fstab /etc/fstab
#fi

echo "20. edit boot options (Rasberry Pi only)"
if [ "$OS" = "R" ]
then
  cp /boot/firmware/cmdline.txt /thinclient_restore 2>> $SOURCE/install.err
  # Use no swap, disable Raspberry pi logi, disable kernel output,
  # mousepoll to improve the mouse speed  
  sed 's/$/ noswap logo.nologo consoleblank=0 net.ifnames=0 loglevel=1 quiet usbhid.mousepoll=0/' -i /boot/firmware/cmdline.txt
  echo "9.9.9.9" > /data/user/thinclient/config/resolv
  chown thinclient:thinclient /data/user/thinclient/config/resolv
fi

echo "21. config monitor and sound (Rasberry Pi only)"
if [ "$OS" = "R" ]
then
  cp /boot/firmware/config.txt /thinclient_restore 2>> $SOURCE/install.err
  sed 's/^#disable_overscan/disable_overscan/' -i /boot/firmware/config.txt
  # set Raspberry Pi to HDMI mode even if no HDMI monitor is detected
  sed 's/^#hdmi_force_hotplug=/hdmi_force_hotplug=/' -i /boot/firmware/config.txt
  # send sound to hdmi 
  sed 's/^#hdmi_drive=/hdmi_drive=/' -i /boot/firmware/config.txt
  # enable monitor stand by
  echo "hdmi_blanking=1" >> /boot/firmware/config.txt
  # disable Raspberry Pi color test and use splashscreen
  echo "disable_splash=1" >> /boot/firmware/config.txt
fi

echo "22. disable GL driver (Raspberry Pi 4 only)"
if [ "$OS" = "R" ] && [ "$MODEL" = "PI4" ]
then
  # Raspberry Pi 4: disable GL driver
  sed 's/^dtoverlay=/#dtoverlay=/' -i /boot/firmware/config.txt 2>> $SOURCE/install.err
fi

echo "23. install software for argon1 case (Raspberry Pi 4 only)"
if [ "$OS" = "R" ]
then
  $SOURCE/desktop/argon1.sh
  systemctl disable argononed
fi

echo "24. configure usb automount"
sed 's/^FS_MOUNTOPTIONS=/FS_MOUNTOPTIONS="-fstype=vfat,user=000,dmask=0000,fmask=0000,gid=thinclient,uid=thinclient"/' -i /etc/usbmount/usbmount.conf 2>> $SOURCE/install.err
# mount device only on USB0
sed 's/^MOUNTPOINTS=.*$/MOUNTPOINTS="\/media\/usb0"/g' -i /etc/usbmount/usbmount.conf 2>> $SOURCE/install.err
sed '/\/media\/usb4/d' -i /etc/usbmount/usbmount.conf 2>> $SOURCE/install.err
# deactivate automount
mv /etc/usbmount/usbmount.conf /etc/usbmount/usbmount.conf.off
# allow USB mount for all users
if [ "$OS" = "U" ]
then
  sed -i 's/^PrivateMounts=yes/PrivateMounts=no/' /lib/systemd/system/systemd-udevd.service 2>> $SOURCE/install.err
fi

echo "25. configure dual monitors (Raspberry Pi 4 only)"
if [ "$OS" = "R" ] && [ "$MODEL" = "PI4" ]
then
  echo "dtoverlay=vc4-fkms-v3d" >> /boot/firmware/config.txt
  #mv /usr/share/dispsetup.sh /data/thinclient
  #chown thinclient:thinclient /data/user/thinclient/dispsetup.sh
  #ln -s /data/user/thinclient/dispsetup.sh /usr/share
fi

echo "26. delete user pi (Rasberry Pi only)"
if [ "$OS" = "R" ]
then
  deluser pi 2>> $SOURCE/install.err
  rm -rf /home/pi
fi

echo "27. configure system"
# set /etc/resolv.conf
#if [ "$OS" = "R" ]
#then
#  sed -e 's/^nameserver.*$/nameserver 9.9.9.9/' -i /etc/resolv.conf
#  sed -e '/^domain/d' -i /etc/resolv.conf
#fi
# Reduce waiting time to end process
echo "DefaultTimeoutStopSec=6s" >> /etc/systemd/system.conf
echo "DefaultTimeoutStopSec=6s" >> /etc/systemd/user.conf
# disable ntp time
timedatectl set-ntp false

echo "28. compile bin"
$SOURCE/compile.sh

echo "29. install xfreerdp3"
if [ "$OS" = "R" ]
then 
  echo "XDG_SESSION_TYPE=x11 ; flatpak run com.freerdp.FreeRDP \$@" > /usr/bin/xfreerdp3
  chmod u=rwx,g=rx,o=rx /usr/bin/xfreerdp3 >> $SOURCE/install.out 2>> $SOURCE/install.out
  #flatpak remote-add --if-not-exists flathub https://flathub.org/repo/flathub.flatpakrepo >> $SOURCE/install.out 2>> $SOURCE/install.err
  #flatpak -y install com.freerdp.FreeRDP >> $SOURCE/install.out 2>> $SOURCE/install.err
  flatpak -y install $SOURCE/xfreerdp3/pi/com.freerdp.FreeRDP.flatpak >> $SOURCE/install.out 2>> $SOURCE/install.err
else
  if [ "$OS" = "U" ] && [ "$MODEL" = "23" ]
  then
    dpkg -i $SOURCE/xfreerdp3/ubuntu/freerdp_2310-amd64.deb >> $SOURCE/install.out
  else
    dpkg -i $SOURCE/xfreerdp3/ubuntu/freerdp-amd64.deb >> $SOURCE/install.out
  fi
  apt-get install -y -f >> $SOURCE/install.out 2>> $SOURCE/install.err
  ln -s /opt/freerdp-nightly/bin/xfreerdp3 /usr/bin/xfreerdp3 >> $SOURCE/install.out 2>> $SOURCE/install.err
fi

echo "30. config plymouth (splashscreen)"
if [ "$OS" = "R" ]
then
  sed 's/$/ splash quiet plymouth.ignore-serial-consoles logo.nologo vt.global_cursor_default=0/' -i /boot/firmware/cmdline.txt
  #echo " splash quiet plymouth.ignore-serial-consoles logo.nologo vt.global_cursor_default=0" > /tmp/cmdline.txt
  #cat /boot/firmware/cmdline.txt /tmp/cmdline.tmp > /tmp/cmdline.txt
  #tr -d "\n" < /tmp/cmdline.txt > /boot/firmware/cmdline.txt
  cp plymouth/bgrt.plymouth /usr/share/plymouth/themes/bgrt
  cp plymouth/splashscreen.png /usr/share/plymouth/themes/spinner/background-tile.png
  cp plymouth/watermark.png /usr/share/plymouth/themes/spinner/watermark.png
  update-alternatives --install /usr/share/plymouth/themes/default.plymouth default.plymouth /usr/share/plymouth/themes/bgrt/bgrt.plymouth 100 >> $SOURCE/install.out 2>> $SOURCE/install.out
  plymouth-set-default-theme bgrt >> $SOURCE/install.out 2>> $SOURCE/install.out
  update-initramfs -u >> $SOURCE/install.out 2>> $SOURCE/install.out
else
  sed -e 's/^GRUB_CMDLINE_LINUX=.*$/GRUB_CMDLINE_LINUX="quiet splash"/g' -i /etc/default/grub 2>> $SOURCE/install.err
  update-grub >> $SOURCE/install.out 2>> $SOURCE/install.out
  cp plymouth/bgrt.plymouth /usr/share/plymouth/themes/bgrt
  cp plymouth/background-tile.png /usr/share/plymouth/themes/spinner/background-tile.png
  cp plymouth/watermark.png /usr/share/plymouth/themes/spinner/watermark.png
  cp plymouth/watermark.png /usr/share/plymouth/ubuntu-logo.png
  update-alternatives --install /usr/share/plymouth/themes/default.plymouth default.plymouth /usr/share/plymouth/themes/bgrt/bgrt.plymouth 100 >> $SOURCE/install.out 2>> $SOURCE/install.err
  update-initramfs -u >> $SOURCE/install.out 2>> $SOURCE/install.out
fi

#echo "32. install driver for pi desktop case (Raspbery Pi 3 only)"
#if [ "$OS" = "R" ]
#then
#  dpkg -i /opt/thinclient/desktop/pidesktop-base.deb >> $SOURCE/install.out 2>> $SOURCE/install.err
#  echo " " >> /boot/firmware/config.txt
#  echo " " >> /boot/firmware/config.txt
#  cp $SOURCE/desktop/pidesktop.sh $DESTINATION/desktop
#  cp $SOURCE/desktop/pidesktop-powerkey.service /lib/systemd
#  cp $SOURCE/desktop/pidesktop-reboot.service /lib/systemd
#  cp $SOURCE/desktop/pidesktop-shutdown.service /lib/systemd
#  cp $SOURCE/desktop/pidesktop-powerkey.service /etc/systemd/system
#  cp $SOURCE/desktop/pidesktop-reboot.service //etc/systemd/system
#  cp $SOURCE/desktop/pidesktop-shutdown.service /etc/systemd/system
#  chown root:root $DESTINATION/desktop/pidesktop.sh
#  chown root:root /lib/systemd/pidesktop-powerkey.service
#  chown root:root /lib/systemd/pidesktop-reboot.service
#  chown root:root /lib/systemd/pidesktop-shutdown.service
#  chown root:root /etc/systemd/system/pidesktop-powerkey.service
#  chown root:root /etc/systemd/system/pidesktop-reboot.service
#  chown root:root /etc/systemd/system/pidesktop-shutdown.service
#  systemctl enable pidesktop-powerkey >> $SOURCE/install.out 2>> $SOURCE/install.err
#  systemctl enable pidesktop-reboot >> $SOURCE/install.out 2>> $SOURCE/install.err
#  systemctl enable pidesktop-shutdown >> $SOURCE/install.out 2>> $SOURCE/install.err
#fi

echo "31: Disable Updates (only Ubuntu)"
if [ "$OS" = "U" ]
then
  echo 'APT::Periodic::Update-Package-Lists "0";' > /etc/apt/apt.conf.d/20auto-upgrades 2>> $SOURCE/install.err
  echo 'APT::Periodic::Download-Upgradeable-Packages "0";' >> /etc/apt/apt.conf.d/20auto-upgrades 2>> $SOURCE/install.err
  echo 'APT::Periodic::AutocleanInterval "0";' >> /etc/apt/apt.conf.d/20auto-upgrades 2>> $SOURCE/install.err
  echo 'APT::Periodic::Unattended-Upgrade "1";' >> /etc/apt/apt.conf.d/20auto-upgrades 2>> $SOURCE/install.err
fi

echo "32: install resolvconf (Raspberry Pi only)"
if [ "$OS" = "R" ]
then
  apt-get install -y resolvconf >> $SOURCE/install.out 2>> $SOURCE/install.err
  echo "nameserver 9.9.9.9" >> /etc/resolv.conf
  #sed -e 's/^nameserver.*$/nameserver 9.9.9.9/' -i /etc/resolv.conf
  sed -e '/^domain/d' -i /etc/resolv.conf
#  echo "nameserver 9.9.9.9" >> /etc/resolv.conf
fi
apt-get -y autoremove >> $SOURCE/install.out 2>> $SOURCE/install.err

echo "33. overlay file system (Raspberry Pi only)"
if [ "$OS" = "R" ]
then
  raspi-config nonint do_overlayfs 0
  raspi-config nonint do_overlayfs 1
fi

echo "* End installation"
if [ -s $SOURCE/install.err ]
then
    echo "**************************************************"
    echo "*  error installation (see $SOURCE/install.err): *"
    echo "**************************************************"
    cat $SOURCE/install.err
fi

echo "********************************************"
echo "**        Reboot system now               **"
echo "********************************************"
exit
